  /**********************************************************************
  *                    BilinearTransformation2D v1.21                    *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/
  
import java.util.Vector;
import java.util.Locale;

public class BilinearTransformation2D extends Transformation {
  private double standardDeviationA[] = new double[4], standardDeviationB[] = new double[4];
  protected Double omegaX = null, omegaY = null;
  protected double a1=1.0, a2=1.0, a3=1.0, a4=1.0, b1=1.0, b2=1.0, b3=1.0, b4=1.0;
  private char type = 'x';
  private String messages[] = new String[] {
                                "Error in BilinearTransformation2D: ",
                                "X-Parameters could not be calculated error-free!",
                                "Y-Parameters could not be calculated error-free!",
                                "Not enough points!",
                                "No Transformationparameters!"
                              };


  public BilinearTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ) {
    this(PointsS, PointsZ, true);
  }
  
  public BilinearTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm) {
    this(PointsS, PointsZ, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, isL2Norm, "GHM");
  }
  
  public BilinearTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm, String modelType) {
    this(PointsS, PointsZ, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, isL2Norm, modelType);
  }
  
  public BilinearTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, double a1, double a2, double a3, double a4, double b1, double b2, double b3, double b4, boolean isL2Norm, String modelType) {
    this.PointsS = new Vector<Point23>();
    this.PointsZ = new Vector<Point23>();
    for (int i=0; i<PointsS.size(); i++){
      for (int j=0; j<PointsZ.size(); j++){
        if (PointsS.get(i).getPKT().equals(PointsZ.get(j).getPKT()) && PointsS.get(i).isPP()&& PointsZ.get(j).isPP()){
          this.PointsS.add(PointsS.get(i));
          this.PointsZ.add(PointsZ.get(j));
          continue;
        }
      }
    }
    
    this.modelType = modelType;
    this.is3DTransformation = false;
    
    this.a1 = a1;
    this.a2 = a2;
    this.a3 = a3;
    this.a4 = a4;
    
    this.b1 = b1;
    this.b2 = b2;
    this.b3 = b3;
    this.b4 = b4;

    this.isL2Norm = isL2Norm;

    if (this.PointsS.size() < 3){
      this.errMsg += this.messages[0]+"\n"+this.messages[3];
      return;
    }

    boolean isCompleteX = false, isCompleteY = false;

    this.type = 'x';
    if (!(isCompleteX = calculateTransformationParameters() )){
      this.errMsg += this.messages[0]+"\n"+this.messages[1]+"\n";
    }
    this.type = 'y';
    if (!(isCompleteY = calculateTransformationParameters() )){
      this.errMsg += this.messages[0]+"\n"+this.messages[2]+"\n";
    }
    this.isComplete = isCompleteX && isCompleteY;
  }

  @Override
  public Point23 transPoint(Point23 P){
    if (!this.isComplete)
      return null;
    double x = this.a1 + this.a2*P.getX() + this.a3*P.getY() + this.a4*P.getX()*P.getY();
    double y = this.b1 + this.b2*P.getX() + this.b3*P.getY() + this.b4*P.getX()*P.getY();
    return new Point23(P.getPKT(), x,y,null, P.isPP());
  }

  private boolean calculateTransformationParameters(){
    /*
    
    x = a1 + a2*X + a3*Y + a4*X*Y
    y = b1 + b2*X + b3*Y + b4*X*Y
    
    */

    double P[][] = null;
    if (this.modelType.equalsIgnoreCase("GHM")){
      P = new double[3*this.PointsS.size()][3*this.PointsS.size()];
      if (this.type == 'x')
        for (int i=0; i<this.PointsS.size(); i++){
          P[i][i]                                             = this.PointsS.get(i).getWeightX();
          P[this.PointsS.size()+i][this.PointsS.size()+i]     = this.PointsS.get(i).getWeightY();

          P[2*this.PointsS.size()+i][2*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightX();
        }
      else
        for (int i=0; i<this.PointsS.size(); i++){
          P[i][i]                                             = this.PointsS.get(i).getWeightX();
          P[this.PointsS.size()+i][this.PointsS.size()+i]     = this.PointsS.get(i).getWeightY();

          P[2*this.PointsS.size()+i][2*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightY();
        }
     }
     else {
        P = new double[this.PointsS.size()][this.PointsS.size()];
        if (this.type == 'x')
          for (int i=0; i<this.PointsS.size(); i++)
            P[i][i] = this.PointsS.get(i).getWeightX()+this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightX();
        else
          for (int i=0; i<this.PointsS.size(); i++)
            P[i][i] = this.PointsS.get(i).getWeightX()+this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightY();
     }

    this.runs = this.maxruns;
    do {

        double X[] = new double[4];
        if (this.type == 'x') {
           X[0] = this.a1;
           X[1] = this.a2;
           X[2] = this.a3;
           X[3] = this.a4;
        }
        else {
           X[0] = this.b1;
           X[1] = this.b2;
           X[2] = this.b3;
           X[3] = this.b4;
        }


        if (this.maxruns == this.runs) {
          double X0[] = new double[X.length];
          System.arraycopy(X,0,X0,0,X.length);
          if ( (X0 = this.levenbergMarquardt(X0, this.lambda)) != null )
            System.arraycopy(X0,0,X,0,X0.length);
            print_r(X0);
        }

        this.runs--;

        if (this.modelType.equalsIgnoreCase("GHM")){
           MathematicalModel ghm = getMathModel("GHM", this.PointsS, this.PointsZ, X);
           ghm.setCofactorMatrix(P);
           X = calculateTransformationParameters(ghm);
           if (X==null)
             return false;
        }
        else if (this.modelType.equalsIgnoreCase("GMM")){
          MathematicalModel gmm = getMathModel("GMM", this.PointsS, this.PointsZ, X);
          gmm.setCofactorMatrix(P);
          X = this.calculateTransformationParameters(gmm);
          if (X==null)
            return false;
        }


        if (this.type == 'x') {
          this.a1 = X[0];
          this.a2 = X[1];
          this.a3 = X[2];
          this.a4 = X[3];
          System.arraycopy(this.standardDeviation,0,this.standardDeviationA,0,this.standardDeviation.length);
          this.omegaX = this.omega;
        }
        else {
          this.b1 = X[0];
          this.b2 = X[1];
          this.b3 = X[2];
          this.b4 = X[3];
          System.arraycopy(this.standardDeviation,0,this.standardDeviationB,0,this.standardDeviation.length);
          this.omegaY = this.omega;
        }

    }while (max_dx > stopiteration && this.runs > 0 );
    return true;
  }
  
  @Override
  public MathematicalModel getMathModel(String modelType, Vector<Point23> pointsS, Vector<Point23> pointsZ, double X[]){
    double A[][] = new double[pointsS.size()][X.length];
    double B[][] = new double[pointsS.size()][3*pointsS.size()];

    double w[]   = new double[A.length];

    for (int i=0; i<pointsS.size(); i++){
        A[i][0] =  1.0;
        A[i][1] =  pointsS.get(i).getX();
        A[i][2] =  pointsS.get(i).getY();
        A[i][3] =  pointsS.get(i).getX()*pointsS.get(i).getY();

        if (this.type == 'x')
          w[i] = X[0]+X[1]*pointsS.get(i).getX() + X[2]*pointsS.get(i).getY() + X[3]*pointsS.get(i).getX()*pointsS.get(i).getY() - pointsZ.get(i).getX();
        else
          w[i] = X[0]+X[1]*pointsS.get(i).getX() + X[2]*pointsS.get(i).getY() + X[3]*pointsS.get(i).getX()*pointsS.get(i).getY() - pointsZ.get(i).getY();

      if (this.modelType.equalsIgnoreCase("GHM")){
        B[i][3*i+0] =  X[1]+X[3]*pointsS.get(i).getY();
        B[i][3*i+1] =  X[2]+X[3]*pointsS.get(i).getX();
        B[i][3*i+2] = -1.0;
      }
    }

    if (this.modelType.equalsIgnoreCase("GMM"))
      return new MathematicalModel(A, X, w);
    else
      return new MathematicalModel(A, B, X, w);

  }
  
  @Override
  public String toString(){
    String str = new String("");
    if (!this.isComplete || !this.errMsg.isEmpty())
      str += "<hr>"+this.messages[3]+"\n"+this.errMsg+"\n<hr>\n";
    try {
      str += String.format(Locale.ENGLISH, " &Omega;<sub>a</sub> = "+resolutionOmega+"\r\n", this.omegaX);
      str += String.format(Locale.ENGLISH, " a<sub>1</sub> = "+resolutionBilinear+"\r\n", this.a1);
      str += String.format(Locale.ENGLISH, " a<sub>2</sub> = "+resolutionBilinear+"\r\n", this.a2);
      str += String.format(Locale.ENGLISH, " a<sub>3</sub> = "+resolutionBilinear+"\r\n", this.a3);
      str += String.format(Locale.ENGLISH, " a<sub>4</sub> = "+resolutionBilinear+"\r\n", this.a4);

      str += "\r\n";

      str += String.format(Locale.ENGLISH, " &sigma<sub>a1</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationA[0]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>a2</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationA[1]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>a3</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationA[2]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>a4</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationA[3]);

    }catch(Exception err) { }
    try {
      str += "\r\n\r\n";
    
      str += String.format(Locale.ENGLISH, " &Omega;<sub>b</sub> = "+resolutionOmega+"\r\n", this.omegaY);
      str += String.format(Locale.ENGLISH, " b<sub>1</sub> = "+resolutionBilinear+"\r\n", this.b1);
      str += String.format(Locale.ENGLISH, " b<sub>2</sub> = "+resolutionBilinear+"\r\n", this.b2);
      str += String.format(Locale.ENGLISH, " b<sub>3</sub> = "+resolutionBilinear+"\r\n", this.b3);
      str += String.format(Locale.ENGLISH, " b<sub>4</sub> = "+resolutionBilinear+"\r\n", this.b4);

      str += "\r\n";

      str += String.format(Locale.ENGLISH, " &sigma<sub>b1</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationB[0]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>b2</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationB[1]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>b3</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationB[2]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>b4</sub> = "+resolutionStdBilinear+"\r\n", this.standardDeviationB[3]);

    }catch(Exception err) { }
    try {
      for (int i=0; i<this.PointsS.size(); i++)
        str += this.PointsZ.get(i).diff(this.transPoint(this.PointsS.get(i))).toHTML()+"\r\n";
    }catch(Exception err) { }
    return "<pre>"+str+"</pre>";
  }


}

