  /**********************************************************************
  *                         CSVFileReader v1.2                           *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

import java.io.*;
import java.util.*;

public class CSVFileReader {
  private File f = null;
  private Vector<String> LinesOfFile = new Vector<String>();

  public CSVFileReader() { }

  public CSVFileReader(String f) {
    this(new File(f));
  }

  public CSVFileReader(File f) {
    this.f = f;
  }

  public Vector<Point23> getPointsFromFile(String seperator, Integer pktRow, Integer xRow, Integer yRow, Integer zRow){
    if (!this.readSourcefile() || this.LinesOfFile == null || pktRow == null || xRow == null || yRow == null)
      return null;
    pktRow--;
    xRow--;
    yRow--;
    if (zRow != null)
      zRow--;
    Vector<Point23> Points = new Vector<Point23>();
    seperator = seperator.trim().equalsIgnoreCase("")?"\\s+":seperator;
    int maxRow = Math.max(Math.max(xRow,yRow),pktRow);
    if (zRow != null)
      maxRow = Math.max(maxRow,zRow);

    for (int i=0; i<this.LinesOfFile.size(); i++){
      String splitData[] = this.LinesOfFile.get(i).split(seperator);
      if (splitData.length > maxRow){
        for (int j=0; j<splitData.length; j++)
          splitData[j] = splitData[j].replace(',', '.').trim();
        try {
          double x,y,z;
          x = Double.parseDouble( splitData[xRow] );
          y = Double.parseDouble( splitData[yRow] );
          z = 0.0;
          if (zRow != null)
            z = Double.parseDouble( splitData[zRow] );
          Points.add(new Point23(splitData[pktRow].trim(), new Double( x ), new Double( y ), zRow==null?null:new Double( z ), true));
        } catch ( Exception e ){
          //System.err.println(e);
          continue;
        }
      }
    }
    if (Points.size() > 0)
      return Points;
    else
      return null;
  }
  
  private boolean readSourcefile(){
    if (!this.f.exists() || !this.f.isFile() || !this.f.canRead())
      return false;
    try {
      InputStream is = new FileInputStream( this.f );
      BufferedReader br = new BufferedReader(new InputStreamReader(is));
      String line = new String("");
      while (( line = br.readLine()) != null)
        this.LinesOfFile.add( line.trim() );
      br.close();
    }
    catch (Exception err) {
      System.err.println(err);
      return false;
    }
    return true;
  }

}



