  /**********************************************************************
  *                       JAid - Hilfesystem v0.1                        *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.html.HTMLEditorKit;
import java.io.*;
import java.net.URI;
import java.util.ArrayList;

class JAidFiles {
  private String[] desc;
  private File[] src;
  private int index = 0;

  public JAidFiles(String htmlFiles[][]) {
     this.desc = new String[htmlFiles.length];
     this.src = new File[htmlFiles.length];
     for (int i=0; i<htmlFiles.length; i++){
       this.desc[i] = htmlFiles[i][0];
       this.src[i]  = new File(htmlFiles[i][1]);
       if (htmlFiles[i][1].toLowerCase().endsWith("index.htm") || htmlFiles[i][1].toLowerCase().endsWith("index.html"))
         this.index = i;
     }
  }
  
  public String[] getDescriptions() {
     return this.desc;
  }
  
  public File[] getSources() {
     return this.src;
  }
  
  public String getDescription(int i) {
     return this.desc[i];
  }
  
  public File getSourceFromDescription(String s) {
    for (int i=0; i<this.desc.length; i++)
      if (s.equalsIgnoreCase( this.desc[i] ))
         return this.src[i];
    return null;
  }
  
  public String getDescriptionFromSource(String d){
    for (int i=0; i<this.src.length; i++)
      if (d.equalsIgnoreCase( this.src[i].toString() ))
         return this.desc[i];
    return null;
  }

  public File getSource(int i) {
     return this.src[i];
  }
  
  public int getIndex(){
     return this.index;
  }
  
}

public class JAid extends JFrame {
  // Anfang Variablen
  private final String version = "0.1";

  private JTextPane textarea = new JTextPane();

  private JPanel menuPanel = new JPanel();
  private JPanel searchPanel = new JPanel();
  private JTextField keywordField = new JTextField();
  private JButton searchButton = new JButton("Suche");
  
  private JRadioButton[] searchOpt = {
     new JRadioButton( "und", true ),
     new JRadioButton( "oder" ),
     new JRadioButton( "Phrase" )
  };

  private ButtonGroup searchOptGroup = new ButtonGroup();
  
  private JList menueList= new JList();
  
  private JPanel footPanel = new JPanel();
  //private JButton listOfContentButton = new JButton("Inhaltsverzeichnis");
  private JLabel moduleName = new JLabel("<html><body><center style=\"padding:3px;\"><em>J</em>Aid v"+this.version+" &copy; 2007 by Michael L&ouml;sler</center></body></html>");
  
  private JScrollPane textareaScrollPane = new JScrollPane(textarea);
  private JScrollPane menuejScrollPane = new JScrollPane(menueList);
  
  private JAidFiles jaidFiles = null;
  private JSplitPane splitPane = new JSplitPane(
               JSplitPane.HORIZONTAL_SPLIT,
               menuPanel,
                textareaScrollPane
               );
   // Ende Variablen

  public JAid(String title, JAidFiles jaidFiles, String jaidIcon, int frameWidth, int frameHeight) {
    // Frame-Initialisierung
    super(title+ " - powered by JAid");
    /*
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {  System.exit(0); }
    });*/
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    try {
      this.setIconImage(new ImageIcon(getClass().getResource(jaidIcon)).getImage());
    }
    catch ( Exception e ) { };


    this.setSize(frameWidth, frameHeight);
    this.setMinimumSize( new Dimension(frameWidth,frameHeight));
    this.setLocation((screenSize.width - frameWidth) / 2, (screenSize.height - frameHeight) / 2);
    Container cp = getContentPane();

    cp.setLayout( new BorderLayout(5, 5) );
    // Anfang Komponenten
    this.jaidFiles = jaidFiles;
    this.initJAid();

    cp.add(splitPane);
    // Ende Komponenten

    this.setResizable(true);
    this.setVisible(true);

  //  this.splitPane.setDividerLocation(0.3);
  }

  // Anfang Ereignisprozeduren
  
  private void initJAid() {

    this.initTextarea();

    this.initMenu();

    this.splitPane.setOneTouchExpandable(true);
    //this.splitPane.setDividerLocation(175);

  }

  
  private void initTextarea() {
    this.textarea.setEditorKit(new HTMLEditorKit());
    this.textarea.setEditable(false);
    try {
      this.textarea.setPage(getClass().getResource( this.jaidFiles.getSource( this.jaidFiles.getIndex() ).toString() ));
    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(null,"Fehler beim Laden der Hilfedatei "+e.getMessage()+"!" , "Fehler", JOptionPane.ERROR_MESSAGE);
      return;
    }
    textarea.addHyperlinkListener(new HyperlinkListener() {
      public void hyperlinkUpdate(HyperlinkEvent h) {
        if (h.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
          String url = h.getDescription();
          if (url.toLowerCase().startsWith("http://") || url.toLowerCase().startsWith("https://") || url.toLowerCase().startsWith("ftp://")  ) {
            try { Desktop.getDesktop().browse( new URI(url) ); } catch ( Exception e ) { }
          }
          else if (url.toLowerCase().startsWith("mailto:")) {
            try { Desktop.getDesktop().mail(new URI(url)); } catch (Exception e) {}
          }
          else {
            try { textarea.setPage(h.getURL());
                  menueList.setSelectedValue( jaidFiles.getDescriptionFromSource( h.getURL().getFile().substring(h.getURL().getFile().lastIndexOf("/")+1) ),true);
            } catch (Exception e) {}
          }
        }
      }
    });
  }
  
  private void initSearchMenu() {
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.HORIZONTAL;
    c.anchor = GridBagConstraints.CENTER;
    c.weighty = 1;
    c.insets = new Insets(5,10,5,10);

    this.searchPanel.setLayout( gbl );
    c.gridx = 0;
    c.gridy = 0;
    c.gridwidth = 3;
    
    gbl.setConstraints(   this.keywordField, c);
    this.searchPanel.add( this.keywordField );

    c.gridy = 1;
    c.gridwidth = 1;
    for (int i=0; i<this.searchOpt.length; i++) {
      c.gridx = i;
      c.weighty = 0.33;
      this.searchOptGroup.add( this.searchOpt[i] );
      gbl.setConstraints(   this.searchOpt[i], c);
      this.searchPanel.add( this.searchOpt[i] );
    }
    
    c.weighty = 1;
    c.gridwidth = 3;
    c.gridy++;
    c.gridx=0;
    gbl.setConstraints(   this.searchButton, c);
    this.searchPanel.add( this.searchButton );

    this.keywordField.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent evt) {
        keywordKeyPressed(evt);
      }
    });
    
    this.searchButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        searchActionPerformed(evt);
      }
    });

  }

  private String[] getKeywordsAsArray(String s) {
     s = s.replace('+',' ').trim();
     return s.toLowerCase().split("\\s+");
     
  }
  
  private void initFooterMenu() {
    this.footPanel.setLayout( new GridLayout(1, 0) );
    //this.footPanel.add( this.listOfContentButton );
    this.footPanel.add( this.moduleName );
    /*
    this.listOfContentButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        listOfContentjButtonActionPerformed(evt);
      }
    });
    */
  }
  
  private void initMenu() {

    this.initSearchMenu();
    this.initFooterMenu();
    this.menueList.setListData(this.jaidFiles.getDescriptions());
    this.menueList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    this.menueList.setSelectedIndex( this.jaidFiles.getIndex() );
    this.menueList.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent evt) {
        menueMouseClicked(evt);
      }
    });
    this.menuPanel.setLayout( new BorderLayout(5, 15) );
    this.menuPanel.add( this.searchPanel, BorderLayout.NORTH );
    this.menuPanel.add( this.footPanel, BorderLayout.SOUTH );
    this.menuPanel.add( this.menuejScrollPane, BorderLayout.CENTER );

  }
  
  private void searchActionPerformed(ActionEvent evt) {
    this.search(this.keywordField.getText().trim(), this.getSearchOption());
  }

  private void keywordKeyPressed(KeyEvent evt) {
    if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
      this.search(this.keywordField.getText().trim(), this.getSearchOption());
    }
  }

  private void listOfContentjButtonActionPerformed(ActionEvent evt) {
    this.menueList.setListData(this.jaidFiles.getDescriptions());
  }
  
  private void menueMouseClicked(MouseEvent evt) {

    try {
       File src = this.jaidFiles.getSourceFromDescription( this.menueList.getSelectedValue().toString() );
       textarea.setPage(getClass().getResource( src.toString() ));

    }
    catch (Exception e) {
      JOptionPane.showMessageDialog(null,"Fehler beim Laden der Hilfedatei "+e.getMessage()+"!" , "Fehler", JOptionPane.ERROR_MESSAGE);
    }

  }
  
  private String getSearchOption() {
     if (this.searchOpt[1].isSelected())
       return "or";
     if (this.searchOpt[2].isSelected())
       return "phrase";
     return "and";
  }
  
  private void search(String q, String opt) {
     if (q.length() < 3){
       JOptionPane.showMessageDialog(null, "Fehler, der Suchterm muss mindestens drei Zeichen lang sein!", "Fehler", JOptionPane.ERROR_MESSAGE);
       return;
     }
     String keywords[] = null;
     if (opt.equalsIgnoreCase("or") || opt.equalsIgnoreCase("and")) {
        keywords = this.getKeywordsAsArray( q );
     }
     else {
        keywords = new String[1];
        keywords[0] = q;
     }
     
     this.setSearchResult(keywords, opt);
  }
  
  private void setSearchResult(String keywords[], String opt){
     int numChar = 50;
     boolean isFound = false;
     String res = new String("");
     for (int k=0; k<this.jaidFiles.getSources().length; k++){
       String str = this.file2str( this.jaidFiles.getSource(k) );

       for (int i=0, j=0; i<keywords.length; i++) {
         int index = -1;
         if ((index = str.toLowerCase().indexOf( keywords[i] )) != -1) {
           j++;
           int sPos = index-numChar, ePos = index+keywords[i].length()+numChar;
           if (sPos<0)
             sPos = 0;
           if (ePos >= str.length())
             ePos = str.length()-1;
           
           String tmpStr = str.substring( sPos, ePos );
           tmpStr = tmpStr.substring( tmpStr.indexOf(" "), tmpStr.lastIndexOf(" "));
           
           if ((opt.equalsIgnoreCase("or") || opt.equalsIgnoreCase("phrase")) || (opt.equalsIgnoreCase("and") && j == keywords.length)) {
              res += "<li><strong><a href=\""+this.jaidFiles.getSource(k)+"\">"+this.jaidFiles.getDescription(k)+"</a></strong>";
              res += "<br><small>" + tmpStr + "&hellip;</small></li>";
              isFound = true;
              break;
           }
         }
       }
     }
     if (isFound)
       res = "<ul>" + res + "</ul>";
     else
       res = "<strong>Kein Treffer!</strong>";
     this.textarea.setText("<html><body><h1>Suchergebnis</h1>" + res + " <hr><p style=\"text-align:center;\">powered by <a href=\"http://derletztekick.com\"><em>J</em>Aid</a> v"+this.version+"</body></html>");
  }
  
  private boolean isFound(String str, String keywords[], String opt){
     for (int i=0, j=0; i<keywords.length; i++) {
        if (str.toLowerCase().indexOf( keywords[i] ) != -1) {
           j++;
           if (opt.equalsIgnoreCase("or") || opt.equalsIgnoreCase("phrase")) {
              return true;
           }
           else if (opt.equalsIgnoreCase("and") && j == (keywords.length)) {
              return true;
           }
        }
     }
     return false;
  }
  
  private String file2str(File f){
    String str = new String("");
    try {
      InputStream is = getClass().getResourceAsStream("/"+f);
      BufferedReader br = new BufferedReader(new InputStreamReader(is));

      String line;
      while ( (line = br.readLine()) != null) {
        if (line.trim().length() != 0) {
          str += " "+line.trim();
        }
      }
      br.close();
    }
    catch (IOException e) {
      JOptionPane.showMessageDialog(null, "Fehler beim Einlesen der Datei \n" + e.getMessage() + "!", "Fehler", JOptionPane.ERROR_MESSAGE);
      return "";
    }
    
    str = str.replaceAll("<[^>]*>","");
    
    str = str.replaceAll("&nbsp;"," ");
    
    str = str.replaceAll("&ouml;","");
    str = str.replaceAll("&Ouml;","");

    str = str.replaceAll("&auml;","");
    str = str.replaceAll("&Auml;","");

    str = str.replaceAll("&uuml;","");
    str = str.replaceAll("&Uuml;","");

    str = str.replaceAll("&szlig;","");
    return str;
  }
   
  // Ende Ereignisprozeduren


}

