  /**********************************************************************
  *                        KoordTrans3D v5.35b                           *
  ************************************************************************
  * Copyright (C) 2007-09 by Michael Loesler, http//derletztekick.com    *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/


import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.File;
import java.util.Vector;

public class KoordTrans extends JFrame {
  // Anfang Variablen
  protected static final String version = "5.35b";
  private JPanel SoftwareNamejPanel = new JPanel();
  private JScrollPane scrollpaneSettings = new JScrollPane();
  private JScrollPane scrollpanetransSettings = new JScrollPane();
  private JScrollPane scrollpaneWeights = new JScrollPane();
  private JScrollPane scrollpaneResults = new JScrollPane();
  private JLabel SoftwareNamejLabel = new JLabel("KoordTrans3D");
  private JLabel SoftwareDescriptionjLabel = new JLabel("2D & 3D-Multitranformationssoftware");
  public JTabbedPane ToolsjTabbedPane = new JTabbedPane();
  public LanguageChooser lc = new LanguageChooser("de.lng");
  public JMenuBar mainMenu = new JMenuBar();
  public JMenu menuFile = new JMenu(), menuSetting = new JMenu(), menuInfo = new JMenu(), langMenu = new JMenu();
  protected JMenuItem item, opSItem, opZItem, spTransItem;

  private JRadioButtonMenuItem deLangMenu, enLangMenu;
  private ButtonGroup langBG = new ButtonGroup();
  
  public SettingPanel settingP = new SettingPanel(this);
  public TransSettingPanel transSettingP = new TransSettingPanel(this);
  public PasspointPanel passpointPanel = new PasspointPanel(this);
  public ResultPanel resultP = new ResultPanel();
  private JAid jaid = null;
  // Ende Variablen

  public KoordTrans(String title, String Icon, int frameWidth, int frameHeight, File sf) {
    // Frame-Initialisierung
    super(title);
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) { System.exit(0); }
    });
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    try {
      this.setIconImage(new ImageIcon(getClass().getResource(Icon)).getImage());
    }
    catch ( Exception e ) { };

    this.setJMenuBar(mainMenu);

    this.setSize(frameWidth, frameHeight);
    this.setMinimumSize( new Dimension(frameWidth,(int)(0.75*frameHeight)));
    this.setLocation((screenSize.width - frameWidth) / 2, (screenSize.height - frameHeight) / 2);
    Container cp = getContentPane();
    cp.setLayout( new BorderLayout(5, 5) );
    
    // Anfang Komponenten
    SoftwareNamejPanel.setLayout( new GridLayout(2, 1, 0, -10) );
    SoftwareNamejPanel.add( SoftwareNamejLabel );
    SoftwareNamejPanel.add( SoftwareDescriptionjLabel );
    SoftwareNamejLabel.setFont(new Font("serif", 1, 28));
    SoftwareDescriptionjLabel.setFont(new Font("Monospaced", 2, 11));
    SoftwareNamejLabel.setHorizontalAlignment(0);
    SoftwareDescriptionjLabel.setHorizontalAlignment(0);
    SoftwareNamejLabel.setForeground(Color.GRAY);
    SoftwareDescriptionjLabel.setForeground(Color.GRAY);
    
    ToolsjTabbedPane.setBorder( BorderFactory.createEmptyBorder(0, 20, 20, 20) );
    scrollpaneSettings.setViewportView( this.settingP );
    scrollpaneWeights.setViewportView( this.passpointPanel );
    scrollpanetransSettings.setViewportView( this.transSettingP );
    //scrollpaneResults.setViewportView( this.resultP );
    //ToolsjTabbedPane.setMinimumSize(new Dimension(500,450));
    ToolsjTabbedPane.add( scrollpaneSettings, this.lc.getEntry("Tabs", 0) );
    ToolsjTabbedPane.add( scrollpanetransSettings, this.lc.getEntry("Tabs", 1) );
    //ToolsjTabbedPane.add( this.passpointPanel, this.lc.getEntry("Tabs", 2) );
    //ToolsjTabbedPane.add( this.scrollpaneWeights, this.lc.getEntry("Tabs", 2) );
    ToolsjTabbedPane.add( this.resultP, this.lc.getEntry("Tabs", 3) );
    if (!lc.isReady) {
      JOptionPane.showMessageDialog(null, "Sprachdatei konnte nicht geladen werden!", "Information", JOptionPane.INFORMATION_MESSAGE);
      return;
    }

    if (sf != null && sf.exists() && sf.isFile()){
      this.settingP.sourceFile1 = sf;
      this.settingP.sourcefilesTextfields.get(0).setText(sf.getName());
    }
    cp.add( SoftwareNamejPanel, BorderLayout.NORTH);
    cp.add( ToolsjTabbedPane, BorderLayout.CENTER );

    //Menu
    this.menuFile = MenuBuilder.addMenuBarItem( this.mainMenu,  this.lc.getEntry("Menu", 0) );
    this.menuSetting = MenuBuilder.addMenuBarItem( this.mainMenu,  this.lc.getEntry("Menu", 1) );
    this.menuInfo = MenuBuilder.addMenuBarItem( this.mainMenu,  this.lc.getEntry("Menu", 2) );
    
    this.opSItem = MenuBuilder.addMenuItem( this.menuFile, this.lc.getEntry("File",0), 'S', this.settingP.ofAl );
    this.opZItem = MenuBuilder.addMenuItem( this.menuFile, this.lc.getEntry("File",1), 'Z', this.settingP.ofAl );
    this.spTransItem = MenuBuilder.addMenuItem( this.menuFile, this.lc.getEntry("File",2), 'A', this.settingP.sfAl );
    MenuBuilder.addMenuItem( menuFile, "-" );
    this.item = MenuBuilder.addMenuItem( this.menuFile, this.lc.getEntry("File",3), 'E' );
    this.item.addActionListener( new ActionListener() {
      public void actionPerformed( ActionEvent e ) {
        System.exit(0);
      }
    });
    
    ActionListener lcAl = new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        selectLang(evt);
      }
    };
    
    ActionListener hfAl = new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        helpFrame(evt);
      }
    };
    
    this.langMenu = MenuBuilder.addSubMenu(this.menuSetting, this.lc.getEntry("Settings", 0));
    try {
      this.deLangMenu = MenuBuilder.addRadioButtonMenuItem(this.langMenu, this.lc.getEntry("Lang", 0), this.lc.getEntry("Languagefile", 0).equalsIgnoreCase("de.lng"), 'G', lcAl );
      this.enLangMenu = MenuBuilder.addRadioButtonMenuItem(this.langMenu, this.lc.getEntry("Lang", 1), this.lc.getEntry("Languagefile", 0).equalsIgnoreCase("en.lng"), 'B', lcAl );
    }catch (Exception e) { }

    this.langBG.add( this.deLangMenu );
    this.langBG.add( this.enLangMenu );
    
    this.item = MenuBuilder.addMenuItem( this.menuInfo, this.lc.getEntry("Info",0), 'H', hfAl );


    // Ende Komponenten
    //this.pack();
    this.setResizable(true);
    this.setVisible(true);

  }

  // Anfang Ereignisprozeduren
  private void helpFrame(ActionEvent evt) {
    String htmlFiles[][] = {  {"Allgemeines", "index.html" },
                              {"Transformationstypen", "transformationen.html"},
                              {"Formeln", "formeln.html" },
                              {"Literaturverzeichnis", "literaturquellen.html" },
                              {"Lizenz","lizenz.html"} ,
                              {"Impressum", "impressum.html" }
                            };
  
      if (jaid != null){
        jaid.setVisible(false);
        jaid.dispose();
        jaid = null;
      }
      jaid = new JAid(this.SoftwareNamejLabel.getText()+" v"+version+" Information und Hilfe",new JAidFiles(htmlFiles),"jaid_icon.png",750,450);
  }
  
  private void selectLang(ActionEvent evt) {
    boolean hasChanged = false;
    String errHead = this.lc.getEntry("Message", 0);
    String errMsg = this.lc.getEntry("Message", 9);
    String infoHead = this.lc.getEntry("Message", 7);
    String infoMsg = this.lc.getEntry("Message", 8);
    LanguageChooser tmpLC = null;
    if (evt.getSource() == enLangMenu){
      if (!this.lc.getEntry("Languagefile", 0).equals("en.lng")){
        tmpLC  = new LanguageChooser("en.lng",false);
        hasChanged = tmpLC.writeLangFile();
      }
      else
        return;
    }
    else if (evt.getSource() == deLangMenu){
      if (!this.lc.getEntry("Languagefile", 0).equals("de.lng")){
        tmpLC = new LanguageChooser("de.lng",false);
        hasChanged = tmpLC.writeLangFile();
      }
      else
        return;
    }

    if (hasChanged && this.lc.isReady){
      this.lc = tmpLC;
      if (JOptionPane.showOptionDialog(null, infoMsg, infoHead, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE,null,null,null) == JOptionPane.YES_OPTION){
        this.setVisible(false);
        this.dispose();
        new KoordTrans(this.getTitle(), "kt_icon.png", this.getWidth(), this.getHeight(),null);
      }
    }
    else
      JOptionPane.showMessageDialog(null, errMsg, errHead ,JOptionPane.ERROR_MESSAGE);
  }
  // Ende Ereignisprozeduren

  public static void main(String[] args) {
    int fw = 560;
    int fh = 750;
    String ft = "KoordTrans3D - 2D & 3D-Koordinatentransformationen v"+version+" by Michael Lsler [http://derletztekick.com]";
    if (args.length == 1 && new File(args[0]).exists())
      new KoordTrans(ft, "kt_icon.png", fw, fh, new File(args[0]));
    else
      new KoordTrans(ft, "kt_icon.png", fw, fh, null);
  }
}

