  /**********************************************************************
  *                       MathematicalModel v1.0                         *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

public class MathematicalModel extends MatrixOperation {
   private double A[][], B[][], X[], w[], P[][];
   private boolean isGHM = false;
   
   public MathematicalModel(double A[][], double X[], double w[]) {
     this(A,X,w,null);
   }
   
   public MathematicalModel(double A[][], double X[], double w[], double P[][]) {
      this.A = A;
      this.X = X;
      this.w = w;
      this.P = P;
      this.isGHM = false;
   }
   
   public MathematicalModel(double A[][], double B[][], double X[], double w[]) {
      this(A,B,X,w,null);
   }
   
   public MathematicalModel(double A[][], double B[][], double X[], double w[], double P[][]) {
      this.A = A;
      this.B = B;
      this.X = X;
      this.w = w;
      this.P = P;
      this.isGHM = true;
   }
   
   public boolean isGHMModel(){
      return this.isGHM;
   }
   
   public boolean isGMMModel(){
      return !this.isGHM;
   }
   
   public double[][] getJacobiA(){
      return this.A;
   }
   
   public double[][] getJacobiB(){
      return this.B;
   }
   
   public double[] getErrorVector(){
      return this.w;
   }
   
   public double[] getUnknownVector(){
      return this.X;
   }
   
   public double[][] getCofactorMatrix(){
      return this.P;
   }
   
   public void setCofactorMatrix(double P[][]){
      this.P = P;
   }
   

   
   
}

