  /**********************************************************************
  *                       MatrixOperation v4.2                           *
  ************************************************************************
  * Copyright (C) 2006-07 by Michael Loesler, http//derletztekick.com    *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/
import java.util.Locale;
abstract class MatrixOperation {

  private static double mashEPS(){
    double eps=1.0,x=2.0,y=1.0;
    while (y<x) {
      eps *= 0.5;
      x = 1.0+eps;
    }
    return eps;
  }

  private static final double eps = mashEPS();

  public double min (double A[]){
    double m = A[0];
    for (int i=1; i<A.length; i++)
      m = Math.min(m,A[i]);
    return m;
  }

  public double min (double A[][]){
    double m = this.min(A[0]);
    for (int i=1; i<A.length; i++)
      m = Math.min(m,this.min(A[i]));
    return m;
  }

  public double max (double A[]){
    double m = A[0];
    for (int i=1; i<A.length; i++)
      m = Math.max(m,A[i]);
    return m;
  }

  public double max (double A[][]){
    double m = this.max(A[0]);
    for (int i=1; i<A.length; i++)
      m = Math.max(m,this.max(A[i]));
    return m;
  }
  
  public double absMax (double A[]){
    double m = Math.abs(A[0]);
    for (int i=1; i<A.length; i++)
      m = Math.max(m,Math.abs(A[i]));
    return m;
  }

  public double absMax (double A[][]){
    double m = this.absMax(A[0]);
    for (int i=1; i<A.length; i++)
      m = Math.max(m,this.absMax(A[i]));
    return m;
  }
  
  public static double[][] eye(int o){
     return eye(o,o);
  }

  public static double[][] eye(int o, int p){
    double A[][] = new double[o][p];
    int m = Math.min(o,p);
    for (int i=0; i<m; i++)
      A[i][i] = 1.0;
    return A;
  }

  public static double[] zeros(int o){
    double[] M = new double[o];
    for (int i=0; i<o; i++)
      M[i]=0.0;
    return M;
  }

  public static double[][] zeros(int o, int p){
    double[][] M = new double[o][p];
    for (int i=0; i<o; i++)
      for (int j=0; j<p; j++)
        M[i][j]=0.0;
    return M;
  }

  public static double[] ones(int o){
    double[] M = new double[o];
    for (int i=0; i<o; i++)
      M[i]=1.0;
    return M;
  }

  public static double[][] ones(int o, int p){
    double[][] M = new double[o][p];
    for (int i=0; i<o; i++)
      for (int j=0; j<p; j++)
        M[i][j]=1.0;
    return M;
  }
  
  public static double[][] diag(int o, double d){
    double[][] M = new double[o][o];
    for (int i=0; i<o; i++)
      M[i][i]=d;
    return M;
  }

  public static double[][] diag(double[] v){
    double[][] M = new double[v.length][v.length];
    for (int i=0; i<v.length; i++)
      M[i][i]=v[i];
    return M;
  }

  public static double det(double M[][]) {
    if (M.length != M[0].length)
      return 0;
    int n = M.length;
    double d[] = new double[n];
    double det = 1;
    double perm[] = new double[n];
    double LU[][] = new double[n][n];
    for (int i=0; i<n; i++){
      for (int j=0; j<n; j++)
        LU[i][j] = M[i][j];
    }
    for (int i=0; i<n; i++){
      perm[i] = i;
      double zmax = 0.0;
      for (int j=0; j<n; j++)
        zmax = Math.max(zmax, Math.abs(LU[i][j]));
      if (zmax == 0)
        return 0;
      d[i] = zmax;
    }

    for (int i=0; i<n-1; i++){
      double piv = Math.abs(LU[i][i]) / d[i];
      int j0 = i;
      for (int j=i+1; j<n; j++) {
        double tmp = Math.abs( LU[j][i] ) / d[j];
        if (piv < tmp){
          piv = tmp;
          j0 = j;
        }
      }
      if (piv < eps)
        return 0;

      if (j0 != i){
        det *= -1;
        double tmp = perm[j0];
        perm[j0] = perm[i];
        perm[i] = tmp;

        tmp = d[j0];
        d[j0] = d[i];
        d[i] = tmp;

        for (int k=0; k<n; k++){
          tmp = LU[j0][k];
          LU[j0][k] = LU[i][k];
          LU[i][k] = tmp;
        }

      }
      for (int j=i+1; j<n; j++)
        if (LU[j][i] != 0){
          LU[j][i] /= LU[i][i];
          double tmp = LU[j][i];
          for (int k=i+1; k<n; k++)
            LU[j][k] -= tmp*LU[i][k];
        }
    }

    if (Math.abs( LU[n-1][n-1] ) < eps){
      return 0;
    }

    for (int i=0; i<M.length; i++)
      det *= LU[i][i];
    return det;
  }

  public static double abs (double M[]) {
    double R = 0;
    for (int i=0; i<M.length; i++)
      R += M[i]*M[i];
    return Math.sqrt(R);
  }

  public static double[] sum (double d, double M[]) {
    for (int i=0; i<M.length; i++)
      M[i] += d;
    return M;
  }

  public static double[] diff (double d, double M[]) {
    for (int i=0; i<M.length; i++)
      M[i] -= d;
    return M;
  }

  public static double[][] sum (double d, double M[][]) {
    for (int i=0; i<M.length; i++)
      for (int j=0; j<M[i].length; j++)
        M[i][j] += d;
    return M;
  }

  public static double[][] diff (double d, double M[][]) {
    for (int i=0; i<M.length; i++)
      for (int j=0; j<M[i].length; j++)
        M[i][j] -= d;
    return M;
  }

  public static double[][] sum (double M1[][], double M2[][]) {
    return getSumMatrix(M1,M2,'+');
  }

  public static double[][] diff (double M1[][], double M2[][]) {
    return getSumMatrix(M1,M2,'-');
  }

  public static double[] sum (double M1[], double M2[]) {
    return getSumMatrix(M1,M2,'+');
  }

  public static double[] diff (double M1[], double M2[]) {
    return getSumMatrix(M1,M2,'-');
  }

  private static double[] getSumMatrix (double M1[], double M2[], char OperationType){
    if (M1.length != M2.length)
      return null;
    int op = (OperationType == '+'?1:-1);
    double R[] = new double [M1.length];
    for (int i=0; i<M1.length; i++)
      R[i] = M1[i] + op * M2[i];
    return R;
  }

  private static double[][] getSumMatrix (double M1[][], double M2[][], char OperationType){
    if ((M1.length != M2.length) || (M1[0].length != M2[0].length))
      return null;
    int op = (OperationType == '+'?1:-1);
    double R[][] = new double [M1.length][M1[0].length];
    for (int i=0; i<M1.length; i++)
      for (int j=0; j<M2[i].length; j++)
        R[i][j] = M1[i][j] + op * M2[i][j];
    return R;
  }

  public static double[] multi (double d, double[] M){
    for (int i=0; i<M.length; i++)
      M[i] *= d;
    return M;
  }

  public static double[][] multi (double d, double[][] M){
    for (int i=0; i<M.length; i++)
      for (int j=0; j<M[i].length; j++)
        M[i][j] *= d;
    return M;
  }

  public static double[][] multi (double M1[], double M2[], boolean returnScalar){
    if (M1.length != M2.length)
      return null;
    if (returnScalar){
      double R[][] = new double[1][1];
      for (int i=0; i<M1.length; i++)
        R[0][0] += M1[i] * M2[i];
      return R;
    }
    else {
      double M1V[][] = new double[M1.length][1];
      double M2H[][] = new double[1][M2.length];
      for (int i=0; i<M1.length; i++) {
        M1V[i][0] = M1[i];
        M2H[0][i] = M2[i];
      }
      return multi(M1V,M2H);
    }

  }

  public static double[] multi (double M1[][], double M2[]){
    if (M1[0].length != M2.length)
      return null;
    double R[] = new double[M1.length];
    for (int i=0; i<M1.length; i++)
      for (int j=0; j<M2.length; j++)
        R[i] += M1[i][j] * M2[j];
    return R;
  }

  public static double[] multi (double M1[], double M2[][]){
    if (M1.length != M2.length)
      return null;
    double R[] = new double[M2[0].length];
    for (int i=0; i<M2[0].length; i++)
      for (int j=0; j<M1.length; j++)
        R[i] += M1[j] * M2[j][i];
    return R;
  }

  public static double[][] multi (double M1[][], double M2[][]){
    if (M1[0].length != M2.length)
      return null;
    double R[][] = new double[M1.length][M2[0].length];
    for (int i=0; i<M1.length; i++)
      for (int j=0; j<M2[0].length; j++)
        for (int k=0; k<M1[0].length; k++)
          R[i][j] += M1[i][k] * M2[k][j];
    return R;
  }

  public static double[][] trans(double M[][]){
    double MT[][] = new double [M[0].length][M.length];
    for (int i=0; i<M.length; i++)
      for (int j=0; j<M[i].length; j++)
        MT[j][i] = M[i][j];
    return MT;
  }

  public static double[][] inv(double M[][]) {
    if (M.length != M[0].length)
      return null;
    int n = M.length;
    double spur=0.0, normA=0.0, normB=0.0;
    int permx[] = new int[n];
    int permy[] = new int[n];
    double invM[][] = new double[n][n];
    for (int i=0; i<n; i++) {
      permx[i] = -1;
      permy[i] = -1;
      for (int j=0; j<n; j++)
        invM[i][j] = M[i][j];
    }

    for (int i=0; i<n; i++){
      double piv = 0.0;
      int nx=0, ny=0;
      for (int ix=0; ix<n; ix++)
        if (permx[ix] == -1) {
          for (int iy=0; iy<n; iy++)
            if (permy[iy] == -1 && Math.abs(piv) < Math.abs(invM[ix][iy])) {
              piv = invM[ix][iy];
              nx = ix;
              ny = iy;
            }
        }

        if (Math.abs(piv) == 0.0){
          System.out.println("Error: Matrix is singular to working precision... " + Math.abs(piv));
          return null;
        }
        else if (Math.abs(piv) < eps) {
          System.out.println("Warning: Matrix is close to singular or badly scaled... "+ Math.abs(piv));
        }

      permx[nx] = ny;
      permy[ny] = nx;
      double tmp = 1.0 / piv;
      for (int j=0; j<n; j++)
        if (j!=nx) {
          double faktor = invM[j][ny] * tmp;
          for (int k=0; k<n; k++)
            invM[j][k] -= invM[nx][k] * faktor;
          invM[j][ny] = faktor;
        }
      for (int j=0; j<n; j++)
        invM[nx][j] *= -tmp;
      invM[nx][ny] = tmp;
    }

    for (int i=0; i<n; i++){
      for (int j=i; j<n; j++){
        if (permx[j]==i){
          for (int k=0; k<n; k++){
            double tmp = invM[i][k];
            invM[i][k] = invM[j][k];
            invM[j][k] = tmp;
          }
          permx[j] = permx[i];
          permx[i] = i;
          break;
        }
      }

      for (int j=i; j<n; j++){
        if (permy[j]==i){
          for (int k=0; k<n; k++){
            double tmp = invM[k][i];
            invM[k][i] = invM[k][j];
            invM[k][j] = tmp;
          }
          permy[j] = permy[i];
          permy[i] = i;
          break;
        }
      }
    }

//    for (int i=0; i<n; i++){
//      double h1=0, h2=0;
//      for (int j=0; j<n; j++){
//        double tmp = 0;
//        for (int k=0; k<n; k++)
//          tmp += M[i][k]*invM[k][j];
//        spur += Math.abs(tmp);
//        h1 += Math.abs(M[i][j]);
//        h2 += Math.abs(invM[i][j]);
//      }
//      normA = Math.max(h1, normA);
//      normB = Math.max(h2, normB);
//      double cond = normA*normB;
//    }
//    spur -= n;
    return invM;
  }

  public static double[] solve(double[][] A, double[] b){
    if (A.length != A[0].length)
      return null;
    double [][] invA = inv(A);
    if (invA == null)
      return null;
    return multi(invA,b);
  }

  public static double[] diag(double[][] A){
    double dA[] = new double[Math.min(A.length, A[0].length)];
    for (int i=0; i<dA.length; i++)
      dA[i] = A[i][i];
    return dA;
  }

  public static double[][] pinv(double[][] G){
    boolean transpose = false;
    int m = G.length, n = G[0].length, r=0;

    double A[][] = null;

    if (n<m) {
      transpose=true;
      A = multi(G, trans(G));
      n=m;
    }
    else {
      A = multi(trans(G),G);
    }
    double dA[] = diag(A);
    double tol = Math.abs(dA[0]*1000);
    for (int i=0; i<dA.length; i++)
      if (dA[i]>0)
        tol = Math.min(tol,dA[i]);
    tol *= 1E-9;
    double tmpL[][] = zeros(A.length, A[0].length);

    for (int k=0; k<n; k++, r++){
      double ll[] = null;
      if (r>0){
        double l1[][] = zeros(n-k,r);
        double l2[]   = zeros(r);
        for (int i=k, l=0; i<n; i++, l++){
          for (int j=0; j<r; j++){
            l1[l][j] = tmpL[i][j];
            l2[j] = tmpL[k][j];
          }
        }
        ll = multi(l1, l2);
      }
      else
        ll = zeros(n-k);

      double a[] = zeros(n-k);
      for (int i=k, l=0; i<n; i++, l++)
        a[l] = A[i][k];

      double all[] = diff(a,ll);

      for (int i=k, l=0; i<n; i++,l++){
        tmpL[i][r] = all[l];
      }

      if (tmpL[k][r]>tol){
        tmpL[k][r]=Math.sqrt(tmpL[k][r]);
        if (k<n){
          for (int i=k+1; i<n; i++){
            tmpL[i][r] /= tmpL[k][r];
          }
        }
      }
      else {
        r--;
      }
    }

    double L[][] = zeros(A.length, r);
    for (int i=0; i<tmpL.length; i++){
      for (int j=0; j<r; j++){
        L[i][j] = tmpL[i][j];
      }
    }

   double M[][]=inv(multi(trans(L),L));
   if (M==null)
     return null;
   if (transpose)
     return multi(trans(G),multi(L,multi(M,multi(M,trans(L)))));
   else
     return multi(L,multi(M,multi(M,multi(trans(L),trans(G)))));
  }

  public static double[] cross(double a[], double b[]){
    if (a.length != 3 || b.length != 3)
      return null;
    double v[] = new double[3];
    v[0] = a[1]*b[2] - a[2]*b[1];
    v[1] = a[2]*b[0] - a[0]*b[2];
    v[2] = a[0]*b[1] - a[1]*b[0];
    return v;
  }

  public static double dot(double a[], double b[]){
    return multi(a, b, true)[0][0];
  }

  public static double vectorAngle(double a[], double b[]) {
    double abs_a = abs(a);
    double abs_b = abs(b);
    if (abs_a == 0 || abs_b == 0)
      return 0;
    return Math.acos(dot(a,b)/(abs_a*abs_b));
  }

  public static double[] unit(double v[]){
    double abs = abs(v);
    if (abs == 0)
      return null;
    for (int i=0; i<v.length; i++)
      v[i] /= abs;
    return v;
  }
  
  public static double norm(double v[], String type){
     double norm = 0.0;
     //L1-Norm
     if (type.equalsIgnoreCase("1")) {
       norm = Math.abs(v[0]);
       for (int i=1; i<v.length; i++)
         norm += Math.abs(v[i]);
     }
     // L(-unendlich) - Norm
     else if (type.equalsIgnoreCase("min")) {
       norm = Math.abs(v[0]);
       for (int i=1; i<v.length; i++)
         norm = Math.min(Math.abs(v[i]),norm);
     }
     //L(+unendlich) - Norm
     else if (type.equalsIgnoreCase("max")) {
       norm = Math.abs(v[0]);
       for (int i=1; i<v.length; i++)
         norm = Math.max(Math.abs(v[i]),norm);
     }
     // L2-Norm
     else {
       norm = v[0]*v[0];
       for (int i=1; i<v.length; i++)
         norm += v[i]*v[i];
       norm = Math.sqrt(norm);
     }
     return norm;
  }

  public static void print_r(double M[]){
    print_r(M,"short");
  }
  
  public static void print_r(double M[], String view){
    for (int i=0; i<M.length; i++)
      if (view.equalsIgnoreCase("short"))
         System.out.println( String.format(Locale.ENGLISH, "%+5.5f",M[i]) );
      else
         System.out.println( String.format(Locale.ENGLISH, "%+5.16f",M[i]) );
  }
  
  public static void print_r(double M[][]){
    print_r(M,"short");
  }

  public static void print_r(double M[][], String view){
    for (int i=0; i<M.length; i++){
      for (int j=0; j<M[i].length; j++)
        if (view.equalsIgnoreCase("short"))
           System.out.print( String.format(Locale.ENGLISH, "%+5.5f  ",M[i][j]) );
        else
           System.out.print( String.format(Locale.ENGLISH, "%+5.16f  ",M[i][j]) );
      System.out.println();
    }
  }

}

