  /**********************************************************************
  *                            MenuBuilder v1.1                          *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/
 
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class MenuBuilder{
  private JMenu fileMenu = new JMenu();
  private JMenu settingsMenu = new JMenu();
  private JMenu helpMenu = new JMenu();
  private JMenu langMenu = new JMenu();
  private JMenu tabMenu = new JMenu();
  
  public JMenuItem openMenu = new JMenuItem();
  public JMenuItem closeMenu = new JMenuItem();
  
  private JRadioButtonMenuItem deLangMenu = new JRadioButtonMenuItem();
  private JRadioButtonMenuItem enLangMenu = new JRadioButtonMenuItem();
  private ButtonGroup langBG = new ButtonGroup();
  
  private JMenuItem tabFormatMenu = new JMenuItem();
  private JMenuItem tabGlobalSettingMenu = new JMenuItem();
  private JMenuItem tabResultMenu = new JMenuItem();
  private JMenuItem tabCalcMenu = new JMenuItem();

  public JMenuItem helpAndInfoMenu = new JMenuItem();

  private LanguageChooser lc = null;
  public JTabbedPane ToolsjTabbedPane = null;


  public static JMenu addMenuBarItem( JMenuBar menuBar, String s ){
    JMenu menu;
    if ( s.indexOf("_") > -1 ){
      int pos = s.indexOf("_");
      char c = s.charAt( pos+1 );
      StringBuffer sb = new StringBuffer(s).delete(pos,pos+1);
      menu =  new JMenu( sb.toString() );
      menu.setMnemonic( c );
    }
    else
      menu = new JMenu( s );

    menuBar.add( menu );
    return menu;
  }
  
  private static JMenuItem processMnemonic( String s ){
    if ( s.indexOf("_") > -1 ){
      int pos = s.indexOf("_");
      char c = s.charAt( pos+1 );
      StringBuffer sb = new StringBuffer(s).delete(pos,pos+1);
      return new JMenuItem( sb.toString(), c );
    }
    else
      return new JMenuItem( s );
  }
  
  public static JMenu addSubMenu(JMenu m, String s){
    JMenu menu;
    if ( s.indexOf("_") > -1 ){
      int pos = s.indexOf("_");
      char c = s.charAt( pos+1 );
      StringBuffer sb = new StringBuffer(s).delete(pos,pos+1);
      menu =  new JMenu( sb.toString() );
      menu.setMnemonic( c );
    }
    else
      menu = new JMenu( s );

    m.add( menu );
    return menu;
  }
  
  public static JRadioButtonMenuItem addRadioButtonMenuItem( JMenu m, String s, boolean isSelected, char keyChar, ActionListener al ) {
    JRadioButtonMenuItem radioItem = new JRadioButtonMenuItem(s,isSelected);

    m.add( radioItem  );
    if ( keyChar != 0 )
      radioItem.setAccelerator(
        KeyStroke.getKeyStroke(keyChar, InputEvent.CTRL_MASK) );

    if ( al != null )
      radioItem.addActionListener( al );

    return radioItem;
  }
  
  public static JRadioButtonMenuItem addRadioButtonMenuItem( JMenu m, String s, char keyChar, ActionListener al ) {
    JRadioButtonMenuItem radioItem = new JRadioButtonMenuItem(s);

    m.add( radioItem  );
    if ( keyChar != 0 )
      radioItem.setAccelerator(
        KeyStroke.getKeyStroke(keyChar, InputEvent.CTRL_MASK) );

    if ( al != null )
      radioItem.addActionListener( al );

    return radioItem;
  }
  
  
  public static JMenuItem addMenuItem( JMenu m, String s, char keyChar, ActionListener al ) {
    if ( s.startsWith("-") ) {
      m.addSeparator();
      return null;
    }

    JMenuItem menuItem = processMnemonic( s );

    m.add( menuItem  );

    if ( keyChar != 0 )
      menuItem.setAccelerator(
        KeyStroke.getKeyStroke(keyChar, InputEvent.CTRL_MASK) );

    if ( al != null )
      menuItem.addActionListener( al );

    return menuItem;
  }

  public static JMenuItem addMenuItem( JMenu m, String s, char c ) {
    return addMenuItem( m, s, c, null );
  }

  public static JMenuItem addMenuItem( JMenu m, String s ) {
    return addMenuItem( m, s, (char)0, null );
  }

  public static JMenuItem addMenuItem( JMenu m, String s, ActionListener al ) {
    return addMenuItem( m, s, (char)0, al );
  }

  
}

