  /**********************************************************************
  *                          PPTableModel v1.4                           *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

import javax.swing.table.*;
import javax.swing.*;
import java.util.*;

class PPTableModel extends AbstractTableModel {
  private int row_length, col_length;
  private Hashtable<String,Object> data;
  private String[] ppcolheads = {"Pkt", "SPx", "SPy", "SPz", "ZPx", "ZPy", "ZPz", "PP"};
  private boolean is3DPoint = false;
  public PPTableModel(){ }

  public PPTableModel(int row_length, boolean is3DPoint){
    this.row_length = row_length;
    this.col_length = ppcolheads.length;
    this.data = new Hashtable<String,Object>();
    this.is3DPoint = is3DPoint;
    for (int i=0; i<=row_length; i++){
      this.data.put("[" + i + ",0]", new String(""));
      this.data.put("[" + i + ",1]", new String(""));
      this.data.put("[" + i + ",2]", new String(""));
      this.data.put("[" + i + ",3]", new String(""));
      this.data.put("[" + i + ",4]", new String(""));
      this.data.put("[" + i + ",5]", new String(""));
      this.data.put("[" + i + ",6]", new String(""));
      this.data.put("[" + i + ",7]", new Boolean(false));
    }
  }
  
  public double String2Double(String item, double elseval){
    double val=elseval;
    if (item != null){
      try {
        val = Double.parseDouble(item);
      }
      catch (NumberFormatException er) {
        val = elseval;
      }
    }
    return val;
  }
  
  public boolean String2Boolean(String item, boolean elseval){
    boolean val = elseval;
    if (item != null){
      try {
        val = Boolean.parseBoolean(item);
      }
      catch (NumberFormatException er) {
        val = elseval;
      }
    }
    return val;
  }

  public int getRowCount(){
    return this.row_length;
  }

  public int getColumnCount() {
    return this.col_length;
  }
  
  public boolean isCellEditable(int rowIndex, int columnIndex) {
    if ((columnIndex == 0) || (!this.is3DPoint && columnIndex == 3) || (!this.is3DPoint && columnIndex == 6)){
    //if (columnIndex == 0 || columnIndex == 3 || columnIndex == 6){
      return false;
    }
    else {
      return true;
    }
  }

  public void setValueAt(Object aValue, int rowIndex, int columnIndex){
    String key = "[" + rowIndex + "," + columnIndex + "]";
    String value = aValue.toString();

    if (value.length() <= 0) {
      this.data.remove(key);
    }

    else if (columnIndex == 0){
      this.data.put(key, new String(value));
    }
    
    else if (columnIndex == 1 || columnIndex == 2 || columnIndex == 3 || columnIndex == 4 || columnIndex == 5 || columnIndex == 6){
      this.data.put(key, new Double(Math.abs(String2Double(value,1))).toString());
    }
    
    else if (columnIndex == 7){
      this.data.put(key, new Boolean(String2Boolean(value,false)));
    }

  }
  
  public Object getValueAt(int rowIndex, int columnIndex){
    String key = "[" + rowIndex + "," + columnIndex + "]";
    return this.data.get(key);

  }

  public Class getColumnClass(int columnIndex){
    /*
    switch(columnIndex){
      case 0: return String.class;
      case 7: return Boolean.class;
      default: return Double.class;
    } */
    switch(columnIndex){
      case 7: return Boolean.class;
      default: return String.class;
    }
  }

  public String getColumnName(int columnIndex){
    return ppcolheads[columnIndex];
  }
}
