  /**********************************************************************
  *                FiveParameterTransformation2D v1.21                   *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

import java.util.Vector;
import java.util.Locale;

public class PolynomialTransformation2D extends Transformation {
  private Vector<Point23> Polynom = new Vector<Point23>();
  int coeffX = 0, coeffY = 0;
  private double[] standardDeviationA = null, standardDeviationB = null;
  protected Double omegaX = null, omegaY = null;
  protected double[] aPar = null, bPar = null;
  private char type = 'x';
  private String messages[] = new String[] {
                                "Error in PolynomialTransformation2D: ",
                                "X-Parameters could not be calculated error-free!",
                                "Y-Parameters could not be calculated error-free!",
                                "Not enough points!",
                                "No Transformationparameters!"
                              };

  public PolynomialTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ) {
    this(PointsS, PointsZ, true);
  }
  
  public PolynomialTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm) {
    this(PointsS, PointsZ, isL2Norm, "GHM");
  }
  
  public PolynomialTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm, String modelType) {
    this(PointsS, PointsZ, -1, -1, isL2Norm, modelType);
  }
  
  public PolynomialTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, int coeffX, int coeffY, boolean isL2Norm, String modelType) {
    this.PointsS = new Vector<Point23>();
    this.PointsZ = new Vector<Point23>();
    for (int i=0; i<PointsS.size(); i++){
      for (int j=0; j<PointsZ.size(); j++){
        if (PointsS.get(i).getPKT().equals(PointsZ.get(j).getPKT()) && PointsS.get(i).isPP()&& PointsZ.get(j).isPP()){
          this.PointsS.add(PointsS.get(i));
          this.PointsZ.add(PointsZ.get(j));
          continue;
        }
      }
    }
    
    this.modelType = modelType;
    this.is3DTransformation = false;

    this.isL2Norm = isL2Norm;
    if (coeffX<0 || coeffX>=this.PointsS.size())
      this.coeffX = this.PointsS.size();
    else
      this.coeffX = coeffX;

    if (coeffY<0 || coeffY>=this.PointsS.size())
      this.coeffY = this.PointsS.size();
    else
      this.coeffY = coeffY;

    if (this.PointsS.size() < 3){
      this.errMsg += this.messages[0]+"\n"+this.messages[3]+"\n";
      return;
    }
    
    this.aPar = ones(this.coeffX);
    this.bPar = ones(this.coeffY);

    boolean isCompleteX = false, isCompleteY = false;

    this.type = 'x';
    if (!(isCompleteX = calculateTransformationParameters() )){
      this.errMsg += this.messages[0]+"\n"+this.messages[1]+"\n";
    }
    this.type = 'y';
    if (!(isCompleteY = calculateTransformationParameters() )){
      this.errMsg += this.messages[0]+"\n"+this.messages[2]+"\n";
    }
    this.isComplete = isCompleteX && isCompleteY;
  }

  private int sum(int a){
    if (a==0)
      return a;
    else
      return a+sum(--a);
  }
  
  private int getIteration(int len){
    int i=0;
    while (this.sum(i)<len)
      i++;
    return i;
  }
  
  private boolean calculateTransformationParameters(){
    /*
    Polynome
            m     m-i
         -----   -----
         \       \
      X = \       \         i      j
       l  /       /  a  *  x  *  y
         /       /    ij    l     l
         -----   -----
          i=0     j=0
          
            m     m-i
         -----   -----
         \       \
      Y = \       \         i      j
       l  /       /  b  *  x  *  y
         /       /    ij    l     l
         -----   -----
          i=0     j=0
    */

    
    double P[][] = null;
    double X[] = null;
    
    if (this.modelType.equalsIgnoreCase("GHM")){
      P = this.getCofactorMatrix();
    }
    else {
      P = new double[this.PointsS.size()][this.PointsS.size()];
      if (this.type == 'x')
        for (int i=0; i<this.PointsS.size(); i++)
          P[i][i] = this.PointsS.get(i).getWeightX()+this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightX();
      else
        for (int i=0; i<this.PointsS.size(); i++)
          P[i][i] = this.PointsS.get(i).getWeightX()+this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightY();

    }


    this.runs = this.maxruns;
    do {

      if (this.type=='x') {
         X = new double[this.aPar.length];
         this.standardDeviationA = new double[this.aPar.length];
         System.arraycopy(this.aPar,0,X,0,this.aPar.length);
      }
      else {
         X = new double[this.bPar.length];
         this.standardDeviationB = new double[this.bPar.length];
         System.arraycopy(this.bPar,0,X,0,this.bPar.length);
      }

      if (this.maxruns == this.runs) {
        double X0[] = new double[X.length];
        System.arraycopy(X,0,X0,0,X.length);
        if ( (X0 = this.levenbergMarquardt(X0, this.lambda)) != null )
          System.arraycopy(X0,0,X,0,X0.length);
      }

      this.runs--;

      if (this.modelType.equalsIgnoreCase("GHM")){
         MathematicalModel ghm = getMathModel("GHM", this.PointsS, this.PointsZ, X);
         ghm.setCofactorMatrix(P);
         X = calculateTransformationParameters(ghm);
         if (X==null)
            return false;
      }
      else if (this.modelType.equalsIgnoreCase("GMM")){
         MathematicalModel gmm = getMathModel("GMM", this.PointsS, this.PointsZ, X);
         gmm.setCofactorMatrix(P);
         X = this.calculateTransformationParameters(gmm);
         if (X==null)
            return false;
      }

      if (this.type=='x'){
         System.arraycopy(X,0,this.aPar,0,X.length);
         System.arraycopy(this.standardDeviation,0,this.standardDeviationA,0,this.standardDeviation.length);
         this.omegaX = this.omega;
      }
      else {
         System.arraycopy(X,0,this.bPar,0,X.length);
         System.arraycopy(this.standardDeviation,0,this.standardDeviationB,0,this.standardDeviation.length);
         this.omegaY = this.omega;
      }


    }while (max_dx > stopiteration && this.runs > 0 );
    return true;
  }

  @Override
  public Point23 transPoint(Point23 P){
    if (!this.isComplete)
      return null;
    double x = 0.0;
    double y = 0.0;
    int m=this.getIteration(this.coeffX);
    for (int i=0, ii=0; i<m; i++){
      for (int j=0; i<m-j; j++, ii++){
        if (ii>this.coeffX-1)
          break;
        x += this.aPar[ii]*Math.pow(P.getX(),i) * Math.pow(P.getY(),j);
      }
      if (ii>this.coeffX-1)
        break;
    }
    m=this.getIteration(this.coeffY);
    for (int i=0, ii=0; i<m; i++){
      for (int j=0; i<m-j; j++, ii++){
        if (ii>this.coeffY-1)
          break;
        y += this.bPar[ii]*Math.pow(P.getX(),i) * Math.pow(P.getY(),j);
      }
      if (ii>this.coeffY-1)
        break;
    }
    return new Point23(P.getPKT(), x,y,null, P.isPP());
  }
  
  @Override
  public MathematicalModel getMathModel(String modelType, Vector<Point23> pointsS, Vector<Point23> pointsZ, double X[]){
    double A[][] = new double[pointsS.size()][X.length];
    double B[][] = new double[pointsS.size()][4*pointsS.size()];

    double w[]   = new double[A.length];

    int u=(this.type=='x')?this.coeffX:this.coeffY;
    int m=this.getIteration(u);

      for (int l=0; l<pointsS.size(); l++){
        int ii=0;
        for (int i=0; i<m; i++){
          for (int j=0; i<m-j; j++){
            if (ii>u-1)
              break;
            A[l][ii] = Math.pow(pointsS.get(l).getX(),i) * Math.pow(pointsS.get(l).getY(),j);
            w[l] += X[ii] * Math.pow(pointsS.get(l).getX(),i) * Math.pow(pointsS.get(l).getY(),j);

            double dX = X[ii] * i*Math.pow(pointsS.get(l).getX(),i-1) * Math.pow(pointsS.get(l).getY(),j);
            double dY = X[ii] * Math.pow(pointsS.get(l).getX(),i) * j*Math.pow(pointsS.get(l).getY(),j-1);

            if (this.modelType.equalsIgnoreCase("GHM")){
              dX = (Double.isNaN(dX))?0.0:dX;
              dY = (Double.isNaN(dY))?0.0:dY;
              B[l][4*l+0] += dX;
              B[l][4*l+1] += dY;
            }
            ii++;
          }
          if (ii>u-1)
            break;
        }
        if (this.modelType.equalsIgnoreCase("GHM")){
          B[l][4*l+2] = -1.0;
          B[l][4*l+3] = -1.0;
        }
        if (this.type=='x')
          w[l] -= pointsZ.get(l).getX();
        else
          w[l] -= pointsZ.get(l).getY();
      }



    if (this.modelType.equalsIgnoreCase("GMM"))
      return new MathematicalModel(A, X, w);
    else
      return new MathematicalModel(A, B, X, w);

  }
  
  
  @Override
  public String toString(){
    String str = new String("");

    if (!this.isComplete || !this.errMsg.isEmpty())
      str += "<hr>"+this.messages[4]+"\n"+this.errMsg+"\n<hr>\n";
    try {
      str += String.format(Locale.ENGLISH, " &Omega;<sub>x</sub>  = "+resolutionOmega+"\r\n", this.omegaX);
      int m=this.getIteration(this.coeffX);
      String tmpStr = new String("");
      for (int i=0, ii=0; i<m; i++){
        for (int j=0; i<m-j; j++, ii++){
          if (ii>this.coeffX-1)
            break;
          str += String.format(Locale.ENGLISH, " a<sub>%d,%d</sub> = "+resolutionPolynomial+"\r\n", i, j, this.aPar[ii]);
          tmpStr += String.format(Locale.ENGLISH, " &sigma<sub>a<sub>%d,%d</sub></sub> = "+resolutionStdPolynomial+"\r\n", i, j, this.standardDeviationA[ii]);
        }
        if (ii>this.coeffX-1)
            break;
      }
      str += "\r\n"+tmpStr+"\r\n\r\n";
    }catch(Exception err) { }
    
    try {
      str += String.format(Locale.ENGLISH, " &Omega;<sub>y</sub>  = "+resolutionOmega+"\r\n", this.omegaY);
      int m=this.getIteration(this.coeffY);
      String tmpStr = new String("");
      for (int i=0, ii=0; i<m; i++){
        for (int j=0; i<m-j; j++, ii++){
          if (ii>this.coeffY-1)
            break;
          str += String.format(Locale.ENGLISH, " b<sub>%d,%d</sub> = "+resolutionPolynomial+"\r\n", i, j, this.bPar[ii]);
          tmpStr += String.format(Locale.ENGLISH, " &sigma<sub>b<sub>%d,%d</sub></sub> = "+resolutionStdPolynomial+"\r\n", i, j, this.standardDeviationB[ii]);
        }
        if (ii>this.coeffY-1)
            break;
      }
      str += "\r\n"+tmpStr;
    }catch(Exception err) { }
    
    try {
      for (int i=0; i<this.PointsS.size(); i++)
        str += this.PointsZ.get(i).diff(this.transPoint(this.PointsS.get(i))).toHTML()+"\r\n";
    } catch(Exception err) { }
    
    return "<pre>"+str+"</pre>";
  }
  
}

