  /**********************************************************************
  *                  ProjectiveTransformation2D v1.21                    *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

import java.util.Vector;
import java.util.Locale;

public class ProjectiveTransformation2D extends Transformation {
  //protected double a1=1.0, a2=1.0, a3=0.0, b1=1.0, b2=1.0, b3=0.0, c1=1.0, c2=1.0;
  
  // Hinweis von Michael Hohm (03. Sep 2007): Startwerte aus "Luhmann" Kap. 4.4.
  protected double a1=1.0, a2=0.0, a3=0.0, b1=0.0, b2=1.0, b3=0.0, c1=0.0, c2=0.0;
  private String messages[] = new String[] {
                                "Error in FiveParameterTransformation2D_2: ",
                                "Parameters could not be calculated error-free!",
                                "Not enough points!",
                                "No Transformationparameters!"
                              };



  public ProjectiveTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ) {
    this(PointsS, PointsZ, true);
  }

  public ProjectiveTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm) {
    this(PointsS, PointsZ, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, isL2Norm, "GHM");
  }

  public ProjectiveTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm, String modelType) {
    this(PointsS, PointsZ, 1.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, isL2Norm, modelType);
  }


  public ProjectiveTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, double a1, double a2, double a3, double b1, double b2, double b3, double c1, double c2, boolean isL2Norm, String modelType) {
    this.PointsS = new Vector<Point23>();
    this.PointsZ = new Vector<Point23>();
    for (int i=0; i<PointsS.size(); i++){
      for (int j=0; j<PointsZ.size(); j++){
        if (PointsS.get(i).getPKT().equals(PointsZ.get(j).getPKT()) && PointsS.get(i).isPP()&& PointsZ.get(j).isPP()){
          this.PointsS.add(PointsS.get(i));
          this.PointsZ.add(PointsZ.get(j));
          continue;
        }
      }
    }
    
    this.modelType = modelType;
    this.is3DTransformation = false;
    
    this.a1 = a1;
    this.a2 = a2;
    this.a3 = a3;

    this.b1 = b1;
    this.b2 = b2;
    this.b3 = b3;

    this.c1 = c1;
    this.c2 = c2;

    this.isL2Norm = isL2Norm;

    if (this.PointsS.size() < 3){
      this.errMsg += this.messages[0]+"\n"+this.messages[2];
      return;
    }

    if (!(this.isComplete = calculateTransformationParameters())){
      this.errMsg += this.messages[0]+"\n"+this.messages[1];
    }
  }
  
  @Override
  public Point23 transPoint(Point23 P){
    if (!this.isComplete)
      return null;
    double x = (this.a1*P.getX() + this.b1*P.getY() + this.c1) / (this.a3*P.getX() + this.b3*P.getY() + 1.0);
    double y = (this.a2*P.getX() + this.b2*P.getY() + this.c2) / (this.a3*P.getX() + this.b3*P.getY() + 1.0);
    return new Point23(P.getPKT(), x,y,null, P.isPP());
  }
  
  private boolean calculateTransformationParameters(){
    /*

        a1*x + b1*y + c1
    X = ----------------
        a3*x + b3*y + 1


        a2*x + b2*y + c2
    Y = ----------------
        a3*x + b3*y + 1
        
    */
    double P[][] = null;

    if (this.modelType.equalsIgnoreCase("GHM")){
      P = this.getCofactorMatrix();
    }
    else {
      P = new double[2*this.PointsS.size()][2*this.PointsS.size()];
      for (int i=0; i<this.PointsS.size(); i++){
        P[i][i]                                         = this.PointsS.get(i).getWeightX()+this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightX();
        P[this.PointsS.size()+i][this.PointsS.size()+i] = this.PointsS.get(i).getWeightX()+this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightY();
      }
    }

    do {

      double X[] = {this.a1,
                    this.a2,
                    this.a3,
                    this.b1,
                    this.b2,
                    this.b3,
                    this.c1,
                    this.c2 };

      if (this.maxruns == this.runs) {
        double X0[] = new double[X.length];
        System.arraycopy(X,0,X0,0,X.length);
        if ( (X0 = this.levenbergMarquardt(X0, this.lambda)) != null )
          System.arraycopy(X0,0,X,0,X0.length);
      }

      this.runs--;

      if (this.modelType.equalsIgnoreCase("GHM")){
         MathematicalModel ghm = getMathModel("GHM", this.PointsS, this.PointsZ, X);
         ghm.setCofactorMatrix(P);
         X = calculateTransformationParameters(ghm);
         if (X==null)
            return false;
      }
      else if (this.modelType.equalsIgnoreCase("GMM")){
         MathematicalModel gmm = getMathModel("GMM", this.PointsS, this.PointsZ, X);
         gmm.setCofactorMatrix(P);
         X = this.calculateTransformationParameters(gmm);
         if (X==null)
            return false;
      }
        
        this.a1 = X[0];
        this.a2 = X[1];
        this.a3 = X[2];
        this.b1 = X[3];
        this.b2 = X[4];
        this.b3 = X[5];
        this.c1 = X[6];
        this.c2 = X[7];
        
    }while (max_dx > stopiteration && this.runs > 0 );
    return true;
  }
  
  @Override
  public MathematicalModel getMathModel(String modelType, Vector<Point23> pointsS, Vector<Point23> pointsZ, double X[]){
    double A[][] = new double[2*pointsS.size()][X.length];
    double B[][] = new double[2*pointsS.size()][4*pointsS.size()];

    double w[]   = new double[A.length];

    for (int i=0; i<pointsS.size(); i++){
        A[i][0] =  pointsS.get(i).getX()/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0);
        A[i][1] =  0.0;
        A[i][2] = -(X[0]*pointsS.get(i).getX()+X[3]*pointsS.get(i).getY()+X[6])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*pointsS.get(i).getX();

        A[i][3] =  pointsS.get(i).getY()/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0);
        A[i][4] =  0.0;
        A[i][5] = -(X[0]*pointsS.get(i).getX()+X[3]*pointsS.get(i).getY()+X[6])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*pointsS.get(i).getY();

        A[i][6] =  1.0/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0);
        A[i][7] =  0.0;


        A[i+pointsS.size()][0] =  0.0;
        A[i+pointsS.size()][1] =  pointsS.get(i).getX()/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0);
        A[i+pointsS.size()][2] = -(X[1]*pointsS.get(i).getX()+X[4]*pointsS.get(i).getY()+X[7])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*pointsS.get(i).getX();

        A[i+pointsS.size()][3] =  0.0;
        A[i+pointsS.size()][4] =  pointsS.get(i).getY()/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0);
        A[i+pointsS.size()][5] = -(X[1]*pointsS.get(i).getX()+X[4]*pointsS.get(i).getY()+X[7])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*pointsS.get(i).getY();

        A[i+pointsS.size()][6] =  0.0;
        A[i+pointsS.size()][7] =  1.0/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0);


        w[i] =                      (X[0]*pointsS.get(i).getX() + X[3]*pointsS.get(i).getY() + X[6]) / (X[2]*pointsS.get(i).getX() + X[5]*pointsS.get(i).getY() + 1.0) - pointsZ.get(i).getX();
        w[i+pointsS.size()] =  (X[1]*pointsS.get(i).getX() + X[4]*pointsS.get(i).getY() + X[7]) / (X[2]*pointsS.get(i).getX() + X[5]*pointsS.get(i).getY() + 1.0) - pointsZ.get(i).getY();

      if (this.modelType.equalsIgnoreCase("GHM")){
        B[i][4*i+0] =  X[0]/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1)-(X[0]*pointsS.get(i).getX()+X[3]*pointsS.get(i).getY()+X[6])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*X[2];
        B[i][4*i+1] =  X[3]/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1)-(X[0]*pointsS.get(i).getX()+X[3]*pointsS.get(i).getY()+X[6])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*X[5];
        B[i][4*i+2] = -1.0;
        B[i][4*i+3] =  0.0;

        B[i+pointsS.size()][4*i+0] =  X[1]/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1)-(X[1]*pointsS.get(i).getX()+X[4]*pointsS.get(i).getY()+X[7])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*X[2];
        B[i+pointsS.size()][4*i+1] =  X[4]/(X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1)-(X[1]*pointsS.get(i).getX()+X[4]*pointsS.get(i).getY()+X[7])/Math.pow((X[2]*pointsS.get(i).getX()+X[5]*pointsS.get(i).getY()+1.0),2)*X[5];
        B[i+pointsS.size()][4*i+2] =  0.0;
        B[i+pointsS.size()][4*i+3] = -1.0;
      }
    }

    if (this.modelType.equalsIgnoreCase("GMM"))
      return new MathematicalModel(A, X, w);
    else
      return new MathematicalModel(A, B, X, w);

  }
  
  @Override
  public String toString(){
    String str = new String("");

    if (!this.isComplete || !this.errMsg.isEmpty())
      str += "<hr>"+this.messages[3]+"\n"+this.errMsg+"\n<hr>\n";
    try {
      str += String.format(Locale.ENGLISH, " &Omega;  = "+resolutionOmega+"\r\n\r\n", this.omega);
      str += String.format(Locale.ENGLISH, " a<sub>1</sub> = "+resolutionProjective+"\r\n", this.a1);
      str += String.format(Locale.ENGLISH, " a<sub>2</sub> = "+resolutionProjective+"\r\n", this.a2);
      str += String.format(Locale.ENGLISH, " a<sub>3</sub> = "+resolutionProjective+"\r\n", this.a3);

      str += String.format(Locale.ENGLISH, " b<sub>1</sub> = "+resolutionProjective+"\r\n", this.b1);
      str += String.format(Locale.ENGLISH, " b<sub>2</sub> = "+resolutionProjective+"\r\n", this.b2);
      str += String.format(Locale.ENGLISH, " b<sub>3</sub> = "+resolutionProjective+"\r\n", this.b3);

      str += String.format(Locale.ENGLISH, " c<sub>1</sub> = "+resolutionProjective+"\r\n", this.c1);
      str += String.format(Locale.ENGLISH, " c<sub>2</sub> = "+resolutionProjective+"\r\n", this.c2);

      str += "\r\n";

      str += String.format(Locale.ENGLISH, " &sigma<sub>a<sub>1</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[0]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>a<sub>2</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[1]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>a<sub>3</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[2]);

      str += String.format(Locale.ENGLISH, " &sigma<sub>b<sub>1</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[3]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>b<sub>2</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[4]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>b<sub>3</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[5]);

      str += String.format(Locale.ENGLISH, " &sigma<sub>c<sub>1</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[6]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>c<sub>2</sub></sub> = "+resolutionStdProjective+"\r\n", this.standardDeviation[7]);

      
      for (int i=0; i<this.PointsS.size(); i++)
        str += this.PointsZ.get(i).diff(this.transPoint(this.PointsS.get(i))).toHTML()+"\r\n";
    }catch(Exception err) { }
    return "<pre>"+str+"</pre>";
  }
}

