  /**********************************************************************
  *                       TransSettingPanel v1.0                         *
  ************************************************************************
  * Copyright (C) 2007 by Michael Loesler, http//derletztekick.com       *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import java.text.*;
public class TransSettingPanel extends JPanel {
  private DecimalFormat intf = (DecimalFormat)DecimalFormat.getInstance(Locale.ENGLISH);
  public Vector<JRadioButton> ct2dTypesJR = new Vector<JRadioButton>();
  public Vector<JRadioButton> ct3dTypesJR = new Vector<JRadioButton>();
  public Vector<JRadioButton> ctOtherTypesJR = new Vector<JRadioButton>();
  public Vector<JFormattedTextField> settingJT = new Vector<JFormattedTextField>();
  public Vector<JLabel> settingJL = new Vector<JLabel>();
  private JPanel ct2dPanel = new JPanel();
  private JPanel ct3dPanel = new JPanel();
  private JPanel ctOtherPanel = new JPanel();
  private JPanel settPanel = new JPanel();
  private ButtonGroup ctBG = new ButtonGroup();
  public ActionListener ttAl2D, ttAl3D;
  protected boolean is3DTransformation = true;
  public KoordTrans p = null;
  
  public TransSettingPanel (KoordTrans parent) {
    this.p = parent;
    intf.applyPattern( "0" );
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    c.fill = GridBagConstraints.HORIZONTAL;
    c.anchor = GridBagConstraints.CENTER;
    c.weightx = 0;
    c.weighty = 0;
    c.insets = new Insets(5,10,5,10);
    int yGrid = 0;
    
    this.ttAl2D = new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        setTransTypeActionPerformed(evt,false);
      }
    };
    
    this.ttAl3D = new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        setTransTypeActionPerformed(evt,true);
      }
    };
    
    
    this.setLayout( gbl );
    this.setBorder( BorderFactory.createEmptyBorder(0, 20, 0, 20) );
    

    this.ct2dPanel.setLayout( gbl );
    this.ct3dPanel.setLayout( gbl );
    this.ctOtherPanel.setLayout( gbl );
    this.settPanel.setLayout( gbl );
    this.ct2dPanel.setBorder( BorderFactory.createTitledBorder( this.p.lc.getEntry("BorderTitle", 0) ));
    this.ct3dPanel.setBorder( BorderFactory.createTitledBorder( this.p.lc.getEntry("BorderTitle", 1) ));
    this.ctOtherPanel.setBorder( BorderFactory.createTitledBorder( this.p.lc.getEntry("BorderTitle", 5) ));
    this.settPanel.setBorder( BorderFactory.createTitledBorder( this.p.lc.getEntry("BorderTitle", 4) ));

    
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 0), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 1), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 2), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 3), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 4), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 5), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 6), false) );
    this.ct2dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation2D", 7), false) );

    this.ct2dTypesJR.get(0).setName("2P2D1");
    this.ct2dTypesJR.get(1).setName("3P2D1");
    this.ct2dTypesJR.get(2).setName("4P2D1");
    this.ct2dTypesJR.get(3).setName("4P2D2");
    this.ct2dTypesJR.get(4).setName("5P2D1");
    this.ct2dTypesJR.get(5).setName("5P2D3");
    this.ct2dTypesJR.get(6).setName("5P2D2");
    this.ct2dTypesJR.get(7).setName("6P2D1");

    this.ct3dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation3D", 0), false) );
    this.ct3dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation3D", 1), false) );
    this.ct3dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation3D", 2), false) );
    this.ct3dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation3D", 3), false) );
    this.ct3dTypesJR.add( new JRadioButton( this.p.lc.getEntry("Transformation3D", 4), true) );
    
    this.ct3dTypesJR.get(0).setName("3P3D1");
    this.ct3dTypesJR.get(1).setName("4P3D1");
    this.ct3dTypesJR.get(2).setName("6P3D1");
    this.ct3dTypesJR.get(3).setName("7P3D1");
    this.ct3dTypesJR.get(4).setName("9P3D1");
    
    this.ctOtherTypesJR.add( new JRadioButton( this.p.lc.getEntry("OtherTransformation", 0), false) );
    this.ctOtherTypesJR.add( new JRadioButton( this.p.lc.getEntry("OtherTransformation", 1), false) );
    this.ctOtherTypesJR.add( new JRadioButton( this.p.lc.getEntry("OtherTransformation", 2), false) );
    this.ctOtherTypesJR.get(0).setName("Poly2D");
    this.ctOtherTypesJR.get(1).setName("Bili2D");
    this.ctOtherTypesJR.get(2).setName("Proj2D");
    

    for (int i=0; i<this.ct2dTypesJR.size(); i++){
      this.ctBG.add( this.ct2dTypesJR.get(i) );
      this.ct2dTypesJR.get(i).addActionListener(this.ttAl2D);
      
      c.gridx = 0;                        // x-Position
      c.gridy = yGrid;                    // y-Position
      c.gridwidth = 1;                    // Breite
      c.gridheight = 1;                   // Hoehe
      yGrid++;
      this.ct2dPanel.add( this.ct2dTypesJR.get(i) );
      gbl.setConstraints( this.ct2dTypesJR.get(i), c);
    }
    
    for (int i=0; i<this.ct3dTypesJR.size(); i++){
      this.ctBG.add( this.ct3dTypesJR.get(i) );
      this.ct3dTypesJR.get(i).addActionListener(this.ttAl3D);
      c.gridx = 0;
      c.gridy = yGrid;
      c.gridwidth = 1;
      c.gridheight = 1;
      yGrid++;
      this.ct3dPanel.add( this.ct3dTypesJR.get(i) );
      gbl.setConstraints( this.ct3dTypesJR.get(i), c);
    }
    
    for (int i=0; i<this.ctOtherTypesJR.size(); i++){
      this.ctBG.add( this.ctOtherTypesJR.get(i) );
      this.ctOtherTypesJR.get(i).addActionListener(this.ttAl2D);
      c.gridx = 0;
      c.gridy = yGrid;
      c.gridwidth = 1;
      c.gridheight = 1;
      yGrid++;
      this.ctOtherPanel.add( this.ctOtherTypesJR.get(i) );
      gbl.setConstraints( this.ctOtherTypesJR.get(i), c);
    }
    
    c.gridx = 0;
    c.gridy = 0;
    c.gridwidth = 1;
    c.gridheight = 1;

    this.settingJT.add( new JFormattedTextField( intf ));
    this.settingJT.add( new JFormattedTextField( intf ));
    this.settingJT.get(0).setText(this.p.lc.getEntry("Texfields", 4));
    this.settingJT.get(1).setText(this.p.lc.getEntry("Texfields", 5));
    this.settingJL.add( new JLabel( this.p.lc.getEntry("Labels", 11)));
    this.settingJL.add( new JLabel( this.p.lc.getEntry("Labels", 12) ));
    for (int i=0; i<this.settingJT.size(); i++){
      c.gridx = 0;
      c.gridy = i;
      c.weightx = 0.0;
      gbl.setConstraints(this.settingJL.get(i), c);
      c.gridx = 1;
      c.weightx = 0.1;
      gbl.setConstraints(this.settingJT.get(i), c);
      this.settPanel.add(this.settingJT.get(i));
      this.settPanel.add(this.settingJL.get(i));
    }

    c.gridx = 0;
    c.gridy = 0;
    gbl.setConstraints(this.ct2dPanel, c);
    this.add( this.ct2dPanel );
    
    c.gridy = 1;
    gbl.setConstraints(this.ct3dPanel, c);
    this.add( this.ct3dPanel );
    
    c.gridy = 2;
    gbl.setConstraints(this.ctOtherPanel, c);
    this.add( this.ctOtherPanel );

    
    c.gridy = 3;
    gbl.setConstraints(this.settPanel, c);
    this.add( this.settPanel );

  }
  
  public String getTransType() {
    for (int i=0; i<this.ct2dTypesJR.size(); i++){
      if (this.ct2dTypesJR.get(i).isSelected())
        return this.ct2dTypesJR.get(i).getName();
    }
    for (int i=0; i<this.ct3dTypesJR.size(); i++){
      if (this.ct3dTypesJR.get(i).isSelected())
        return this.ct3dTypesJR.get(i).getName();
    }
    for (int i=0; i<this.ctOtherTypesJR.size(); i++){
      if (this.ctOtherTypesJR.get(i).isSelected())
        return this.ctOtherTypesJR.get(i).getName();
    }
    return new String("");
  }
  
  public void setTransTypeActionPerformed(ActionEvent evt, boolean is3D) {
    this.is3DTransformation = is3D;
    this.p.settingP.setApproxValueVisible();
    this.p.settingP.csvSettingsSpinner.get(3).setEnabled( this.is3DTransformation );
    this.p.settingP.csvSettingsSpinner.get(7).setEnabled( this.is3DTransformation );
  }
  

  
}

