  /**********************************************************************
  *                        Transformation v2.12                          *
  ************************************************************************
  * Copyright (C) 2007-09 by Michael Loesler, http//derletztekick.com    *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/
import java.util.Vector;
abstract public class Transformation extends MatrixOperation {
  boolean isComplete = false;
  Vector<Point23> PointsS = null;
  Vector<Point23> PointsZ = null;
  double standardDeviation[] = null;
  double M=1.0, Mx=1.0, My=1.0, Mz=1.0, R=0.0, Rx=0.0, Ry=0.0, Rz=0.0, Tx=0.0, Ty=0.0, Tz=0.0;

  boolean isL2Norm = true, is3DTransformation = false;
  String modelType = "GHM", resolutionScale = "%+27.20f", resolutionAngle = "%+27.20f", resolutionTranslation = "%+27.20f";
  String resolutionOmega = "%+20.15f", resolutionStdScale = "%+17.10f", resolutionStdAngle = "%+17.10f", resolutionStdTranslation = "%+17.10f";
  String resolutionPolynomial = "%+27.20f", resolutionBilinear = "%+27.20f", resolutionProjective = "%+27.20f";
  String resolutionStdPolynomial = "%+17.10f", resolutionStdBilinear = "%+17.10f", resolutionStdProjective = "%+17.10f";
  String errMsg = new String("");
  Double omega = null;
  int maxruns = 100, runs = maxruns;
  double stopiteration = 1.0E-10, max_dx = 2147483647.0, lambda = 100.0, l1norm = 1.0E-5;

  public Point23 transPoint(Point23 P){
    return null;
  }
  
  public MathematicalModel getMathModel(String modelType, Vector<Point23> pointsS, Vector<Point23> pointsZ, double X[]){
    return null;
  }
  
  public double[] approxScales(Point23 centroidA, Vector<Point23> pointsA, Point23 centroidB, Vector<Point23> pointsB){
     double x = 0.0, y=0.0, z=0.0, len=pointsA.size();
     for (int i=0; i<len; i++){
        x += Math.abs(centroidA.getX() - pointsA.get(i).getX()) / Math.abs(centroidB.getX() - pointsB.get(i).getX());
        y += Math.abs(centroidA.getY() - pointsA.get(i).getY()) / Math.abs(centroidB.getY() - pointsB.get(i).getY());
        if (this.is3DTransformation)
          z += Math.abs(centroidA.getZ() - pointsA.get(i).getZ()) / Math.abs(centroidB.getZ() - pointsB.get(i).getZ());
     }
     if (this.is3DTransformation)
      return new double[] {Math.abs(x)/len,Math.abs(y)/len,Math.abs(z)/len};
    return new double[] {Math.abs(x)/len,Math.abs(y)/len};
  }
  
  public Point23 approxCentroid(Vector<Point23> points){
    double x=0.0, y=0.0, z=0.0, len=points.size();
    
    for (int i=0; i<len; i++){
       x+=points.get(i).getX().doubleValue();
       y+=points.get(i).getY().doubleValue();
       if (this.is3DTransformation)
         z+=points.get(i).getZ().doubleValue();
    }
    if (this.is3DTransformation)
      return new Point23("C",x/len,y/len,z/len);
    return new Point23("C",x/len,y/len);
  }
  
  public double[] levenbergMarquardt(double X0[], double lamda){
    int v = 2, it = 0;
    MathematicalModel mathModel = this.getMathModel("GMM", this.PointsS, this.PointsZ, X0);
    double A[][]  = mathModel.getJacobiA();
    double X[]    = mathModel.getUnknownVector();
    double w[]    = mathModel.getErrorVector();
    double N[][]  = multi(trans(A), A);
    double grad[] = multi(trans(A), w);
    double I[][]  = this.eye(N.length);
    double e1 = 1.0E-15, e2 = 1.0E-15, e3 = 1.0E-15, mu = Math.abs(lamda); //mu = Math.abs(lamda) * this.max(this.diag(N));
    
    boolean foundSolution = this.norm(grad,"max") <= e1;
    while (!foundSolution && ++it<this.maxruns) {
      //System.out.println(it);
      try {
         double dx[] = multi(multi(-1.0,inv(sum(N,multi(mu,I)))), grad);
         if (dx==null)
            return X;

         if (this.norm(dx,"2")<=e2*(this.norm(X,"2")+e2)) {
            foundSolution = true;
         }
         else {
            MathematicalModel newMathModel = this.getMathModel("GMM", this.PointsS, this.PointsZ, sum(X,dx));
            double wnew[] = newMathModel.getErrorVector();
            double roh = (Math.pow(norm(w,"2"),2) - Math.pow(norm(wnew,"2"),2)) / Math.abs(dot(dx,sum(multi(mu,dx),grad)));

            if (roh > 0){
               A = newMathModel.getJacobiA();
               X = newMathModel.getUnknownVector();
               w = newMathModel.getErrorVector();
               N = multi(trans(A), A);
               grad = multi(trans(A), w);
               foundSolution = norm(grad,"max") <= e1 || norm(w,"2")<=e3;
               mu *= Math.max(1.0/3.0, 1.0-Math.pow(2.0*roh-1.0,3));
               v = 2;
            }
            else {
               mu *= (double)v*10.0;
               v  *= 2;
            }
         }
         
         
         
      } catch (Exception e){
        System.err.println(e);
        return X;
      }
    }
    return X;
  }

  private double[] calculateTransformationParametersGMM(double X[], double w[], double A[][], double P[][]){
    double x[] = new double[X.length];
    try {
      double AT[][] = trans(A);
      double N[][] = multi(trans(A),multi(P,A));

      double invN[][] = inv(N);
      if (invN == null)
        return null;

      invN = multi(-1.0, invN);
      x = multi(invN,multi(trans(A),multi(P,w)));

      double v[] = diff(multi(A,x),w);

      this.omega = new Double( multi(v,multi(P,v),true)[0][0] );
      this.standardDeviation = new double[A[0].length];
      if (A.length-A[0].length>0)
        for (int i=0; i<this.standardDeviation.length; i++)
          this.standardDeviation[i] = Math.sqrt( this.omega / (A.length - A[0].length) * -invN[i][i] );

      X = sum(X,x);
      this.max_dx = absMax(x);
      
      if (!this.isL2Norm){
        double Qll[][] = multi(A,multi(invN,trans(A)));
        double sigmal[] = new double[Qll.length];
        double zz[] = new double[Qll.length];

        for (int i=0; i<Qll.length; i++){
          sigmal[i] =  1.0*Math.sqrt(-Qll[i][i]);
          zz[i] = Math.abs(v[i]/sigmal[i]);
        }

        for (int i=0; i<P.length; i++){
          if (zz[i] < this.l1norm)
            P[i][i] = 1.0;
          else
            P[i][i] *= this.l1norm/zz[i];
        }

        double maxP = this.max(P);

        for (int i=0; i<P.length; i++)
          P[i][i] /= maxP;
      }

    } catch (Exception e){
      System.err.println(e);
      return null;
    }
    return X;
    
  }
  
  public double[] calculateTransformationParameters(MathematicalModel mathModel){
    double A[][] = mathModel.getJacobiA();
    double B[][] = mathModel.getJacobiB();
    double P[][] = mathModel.getCofactorMatrix();
    double X[] = mathModel.getUnknownVector();
    double w[] = mathModel.getErrorVector();

    if (mathModel.isGMMModel())
      return calculateTransformationParametersGMM(X, w, A, P);
    else if (mathModel.isGHMModel())
      return calculateTransformationParametersGHM(X, w, A, P, B);
    return null;
  }
  
  private double[] calculateTransformationParametersGHM(double X[], double w[], double A[][], double P[][], double B[][]){
    double x[]   = new double[X.length];
    double k[]   = new double[A.length];
    try {
      double AT[][] = trans(A);
      double BT[][] = trans(B);
      double Qll[][] = inv(P);
      if (Qll == null)
        return null;

      double BQBT[][] = multi(B,multi(Qll,BT));
      double N[][] = zeros(B.length+X.length,B.length+X.length);
      double kx[] = new double[N.length];

      //erste Zeile
      for (int i=0; i<BQBT.length; i++)
        for (int j=0; j<BQBT[i].length; j++)
          N[i][j] = BQBT[i][j];

      for (int i=0; i<A.length; i++)
        for (int j=0; j<A[i].length; j++)
          N[i][BQBT[i].length+j] = A[i][j];

      //Zweite Zeile
      for (int i=0; i<AT.length; i++)
        for (int j=0; j<AT[i].length; j++)
          N[BQBT.length+i][j] = AT[i][j];

      for (int i=0; i<w.length; i++)
        kx[i] = w[i];

      double invN[][] = inv(N);
      if (invN == null)
        return null;

      invN = multi(-1.0, invN);

      kx = multi(invN, kx);
      for (int i=0; i<kx.length; i++){
        if (i<k.length)
          k[i] = kx[i];
        else if (i<k.length+x.length)
          x[i-k.length] = kx[i];
      }

      double v[] = multi(Qll,multi(BT,k));
      double pvv = multi(v,multi(P,v),true)[0][0];
      this.omega = new Double( sum(pvv, multi(multi(2,k), sum(multi(B,v),sum(multi(A,x),w)),true))[0][0] );

      double Qxx[][] = zeros(X.length, X.length);
      for (int i=0; i<Qxx.length; i++)
        for (int j=0; j<Qxx[i].length; j++)
          Qxx[i][j] = invN[A.length+i][A.length+j];

      this.standardDeviation = new double[A[0].length];
      if (A.length-A[0].length>0)
        for (int i=0; i<this.standardDeviation.length; i++)
          this.standardDeviation[i] = Math.sqrt( pvv / (A.length - A[0].length) * Qxx[i][i] );

      X = sum(X,x);

      this.max_dx = absMax(x);

      if (!this.isL2Norm){

        double Q11[][] = zeros(B.length,B.length);
        for (int i=0; i<B.length; i++)
          for (int j=0; j<B.length; j++)
            Q11[i][j] = invN[i][j];

        double Qvv[][]  = multi(Qll,multi(trans(B),multi(Q11,multi(B, Qll))));
        double Qll2[][] = diff(Qll,Qvv);

        double sigmal[] = new double[Qll.length];
        double zz[] = new double[Qll.length];

        for (int i=0; i<Qll2.length; i++){
          sigmal[i] =  1.0*Math.sqrt(Qll2[i][i]);
          zz[i] = Math.abs(v[i]/sigmal[i]);
        }

        for (int i=0; i<P.length; i++){
          if (zz[i] < this.l1norm)
            P[i][i] = 1.0;
          else
            P[i][i] *= this.l1norm/zz[i];
        }

        double maxP = this.max(P);

        for (int i=0; i<P.length; i++)
          P[i][i] /= maxP;
      }

    } catch (Exception e){
      System.err.println(e);
      return null;
    }
    return X;
  }
  
  public double[][] getCofactorMatrix(){
    double P[][] = null;
    if (!this.is3DTransformation) {
      if (this.modelType.equalsIgnoreCase("GHM")){
        P = new double[4*this.PointsS.size()][4*this.PointsS.size()];
        for (int i=0; i<this.PointsS.size(); i++){
          P[i][i]                                             = this.PointsS.get(i).getWeightX();
          P[this.PointsS.size()+i][this.PointsS.size()+i]     = this.PointsS.get(i).getWeightY();

          P[2*this.PointsS.size()+i][2*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightX();
          P[3*this.PointsS.size()+i][3*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightY();
        }
      }
      else if (this.modelType.equalsIgnoreCase("GMM")){
        P = new double[2*this.PointsS.size()][2*this.PointsS.size()];
        for (int i=0; i<this.PointsS.size(); i++){
          P[i][i]                                             = this.PointsZ.get(i).getWeightX(); //this.PointsS.get(i).getWeightX()+this.PointsZ.get(i).getWeightX();
          P[this.PointsZ.size()+i][this.PointsZ.size()+i]     = this.PointsZ.get(i).getWeightY(); //this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightY();
        }
      }
    }
    else {
      if (this.modelType.equalsIgnoreCase("GHM")){
        P = new double[6*this.PointsS.size()][6*this.PointsS.size()];
        for (int i=0; i<this.PointsS.size(); i++){
          P[i][i]                                             = this.PointsS.get(i).getWeightX();
          P[this.PointsS.size()+i][this.PointsS.size()+i]     = this.PointsS.get(i).getWeightY();
          P[2*this.PointsS.size()+i][2*this.PointsS.size()+i] = this.PointsS.get(i).getWeightZ();

          P[3*this.PointsS.size()+i][3*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightX();
          P[4*this.PointsS.size()+i][4*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightY();
          P[5*this.PointsS.size()+i][5*this.PointsS.size()+i] = this.PointsZ.get(i).getWeightZ();
        }
      }
      else if (this.modelType.equalsIgnoreCase("GMM")){
        P = new double[3*this.PointsS.size()][3*this.PointsS.size()];
        for (int i=0; i<this.PointsS.size(); i++){
          P[i][i]                                             = this.PointsZ.get(i).getWeightX(); //this.PointsS.get(i).getWeightX()+this.PointsZ.get(i).getWeightX();
          P[this.PointsZ.size()+i][this.PointsZ.size()+i]     = this.PointsZ.get(i).getWeightY(); //this.PointsS.get(i).getWeightY()+this.PointsZ.get(i).getWeightY();
          P[2*this.PointsZ.size()+i][2*this.PointsZ.size()+i] = this.PointsZ.get(i).getWeightZ(); //this.PointsS.get(i).getWeightZ()+this.PointsZ.get(i).getWeightZ();
        }
      }
    }
    return P;
  }
  
}

