  /**********************************************************************
  *                TwoParameterTransformation2D v1.51                    *
  ************************************************************************
  * Copyright (C) 2007-09 by Michael Loesler, http//derletztekick.com    *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
  * it under the terms of the GNU General Public License as published by *
  * the Free Software Foundation; either version 3 of the License, or    *
  * (at your option) any later version.                                  *
  *                                                                      *
  * This program is distributed in the hope that it will be useful,      *
  * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
  * GNU General Public License for more details.                         *
  *                                                                      *
  * You should have received a copy of the GNU General Public License    *
  * along with this program; if not, see <http://www.gnu.org/licenses/>  *
  * or write to the                                                      *
  * Free Software Foundation, Inc.,                                      *
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            *
  *                                                                      *
   **********************************************************************/

import java.util.Vector;
import java.util.Locale;

class TwoParameterTransformation2D extends Transformation {
  private String messages[] = new String[] {
                                "Error in TwoParameterTransformation2D: ",
                                "Parameters could not be calculated error-free!",
                                "Not enough points!",
                                "No Transformationparameters!"
                              };

  public TwoParameterTransformation2D(double Rx, double Ry) {
    this.Rx = Rx;
    this.Ry = Ry;
    this.isComplete = true;
  }

  public TwoParameterTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ) {
    this(PointsS, PointsZ, true);
  }
  
  public TwoParameterTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm) {
    this(PointsS, PointsZ, 0.0, 0.0, isL2Norm, "GHM");
  }
  
  public TwoParameterTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, boolean isL2Norm, String modelType) {
    this(PointsS, PointsZ, 0.0, 0.0, isL2Norm, modelType);
  }

  public TwoParameterTransformation2D(Vector<Point23> PointsS, Vector<Point23> PointsZ, double Rx, double Ry, boolean isL2Norm, String modelType) {
    this.PointsS = new Vector<Point23>();
    this.PointsZ = new Vector<Point23>();
    this.Rx = Rx;
    this.Ry = Ry;
    this.modelType = modelType;
    this.is3DTransformation = false;
    for (int i=0; i<PointsS.size(); i++){
      for (int j=0; j<PointsZ.size(); j++){
        if (PointsS.get(i).getPKT().equals(PointsZ.get(j).getPKT()) && PointsS.get(i).isPP()&& PointsZ.get(j).isPP()){
          this.PointsS.add(PointsS.get(i));
          this.PointsZ.add(PointsZ.get(j));
          continue;
        }
      }
    }
    
    this.isL2Norm = isL2Norm;

    if (this.PointsS.size() < 2){
      this.errMsg += this.messages[0]+"\n"+this.messages[2];
      return;
    }

    if (!(this.isComplete = calculateTransformationParameters())){
      this.errMsg += this.messages[0]+"\n"+this.messages[1];
    }
  }

  @Override
  public Point23 transPoint(Point23 P){
    if (!this.isComplete)
      return null;
    double x = Math.cos(this.Rx)*P.getX() - Math.sin(this.Ry)*P.getY();
    double y = Math.sin(this.Rx)*P.getX() + Math.cos(this.Ry)*P.getY();
    return new Point23(P.getPKT(), x,y,null, P.isPP());
  }

  private boolean calculateTransformationParameters(){
    /*
    Rotationsmatrix
           cos(Rx)  -sin(Ry)
           sin(Rx)   cos(Ry)
    */

//    Point23 cS = this.approxCentroid(this.PointsS);
//    Point23 cZ = this.approxCentroid(this.PointsZ);
//    Point23 dCC = cZ.diff(cS);
//    double m[] = approxScales(cS, this.PointsS, cZ, this.PointsZ);

//    this.Rx = 0.0;
//    this.Ry = 0.0;

    double P[][] = this.getCofactorMatrix();

    do {

      double X[] = {this.Rx,
                    this.Ry};

      if (this.maxruns == this.runs) {
        double X0[] = new double[X.length];
        System.arraycopy(X,0,X0,0,X.length);
        if ( (X0 = this.levenbergMarquardt(X0, this.lambda)) != null )
          System.arraycopy(X0,0,X,0,X0.length);
      }

      this.runs--;

      if (this.modelType.equalsIgnoreCase("GHM")){
         MathematicalModel ghm = getMathModel("GHM", this.PointsS, this.PointsZ, X);
         ghm.setCofactorMatrix(P);
         X = calculateTransformationParameters(ghm);
         if (X==null)
            return false;
      }
      else if (this.modelType.equalsIgnoreCase("GMM")){
         MathematicalModel gmm = getMathModel("GMM", this.PointsS, this.PointsZ, X);
         gmm.setCofactorMatrix(P);
         X = this.calculateTransformationParameters(gmm);
         if (X==null)
            return false;
      }
      
      this.Rx = X[0]%(2.0*Math.PI);
      this.Ry = X[1]%(2.0*Math.PI);

    }while (max_dx > stopiteration && this.runs > 0 );
    return true;
  }
  
  @Override
  public MathematicalModel getMathModel(String modelType, Vector<Point23> pointsS, Vector<Point23> pointsZ, double X[]){
    double A[][] = new double[2*pointsS.size()][X.length];
    double B[][] = new double[2*pointsS.size()][4*pointsS.size()];

    double w[]   = new double[A.length];

    for (int i=0; i<pointsS.size(); i++){
        A[i][0] = -Math.sin(X[0])*pointsS.get(i).getX();
        A[i][1] = -Math.cos(X[1])*pointsS.get(i).getY();

        A[i+pointsS.size()][0] =  Math.cos(X[0])*pointsS.get(i).getX();
        A[i+pointsS.size()][1] = -Math.sin(X[1])*pointsS.get(i).getY();

        w[i] =                Math.cos(X[0])*pointsS.get(i).getX() - Math.sin(X[1])*pointsS.get(i).getY() - pointsZ.get(i).getX();
        w[i+pointsS.size()] = Math.sin(X[0])*pointsS.get(i).getX() + Math.cos(X[1])*pointsS.get(i).getY() - pointsZ.get(i).getY();


      if (this.modelType.equalsIgnoreCase("GHM")){
        B[i][4*i+0] =  Math.cos(X[0]);
        B[i][4*i+1] = -Math.sin(X[1]);
        B[i][4*i+2] = -1.0;
        B[i][4*i+3] =  0.0;

        B[i+pointsS.size()][4*i+0] =  Math.sin(X[0]);
        B[i+pointsS.size()][4*i+1] =  Math.cos(X[1]);
        B[i+pointsS.size()][4*i+2] =  0.0;
        B[i+pointsS.size()][4*i+3] = -1.0;
      }
    }

    if (this.modelType.equalsIgnoreCase("GMM"))
      return new MathematicalModel(A, X, w);
    else
      return new MathematicalModel(A, B, X, w);

  }

  @Override
  public String toString(){
    String str = new String("");
    if (!this.isComplete || !this.errMsg.isEmpty())
      str += "<hr>"+this.messages[3]+"\n"+this.errMsg+"\n<hr>\n";
    try {
      str += String.format(Locale.ENGLISH, " &Omega;  = "+resolutionOmega+"\r\n\r\n", this.omega);

      str += String.format(Locale.ENGLISH, " &sigma;<sup>2</sup> = "+resolutionOmega+"\r\n\r\n", this.omega/2.0);
      str += String.format(Locale.ENGLISH, " s<sub>P</sub> = "+resolutionOmega+"\r\n\r\n", Math.sqrt(2.0) * Math.sqrt( Math.abs(this.omega/2.0)) );

      str += String.format(Locale.ENGLISH, " R<sub>x</sub> = "+resolutionAngle+"\r\n", this.Rx);
      str += String.format(Locale.ENGLISH, " R<sub>y</sub> = "+resolutionAngle+"\r\n", this.Ry);

      str += "\n";

      str += String.format(Locale.ENGLISH, " &sigma<sub>Rx</sub> = "+resolutionStdAngle+"\n", this.standardDeviation[0]);
      str += String.format(Locale.ENGLISH, " &sigma<sub>Ry</sub> = "+resolutionStdAngle+"\n", this.standardDeviation[1]);

      for (int i=0; i<this.PointsS.size(); i++)
        str += this.PointsZ.get(i).diff(this.transPoint(this.PointsS.get(i))).toHTML()+"\r\n";

    }catch(Exception err) { System.err.println(err); }
    return "<pre>"+str+"</pre>";
  }
}
